# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: featureform/proto/serving.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1f\x66\x65\x61tureform/proto/serving.proto\x12\x19\x66\x65\x61tureform.serving.proto\"\x15\n\x05Model\x12\x0c\n\x04name\x18\x01 \x01(\t\"}\n\x13TrainingDataRequest\x12\x35\n\x02id\x18\x01 \x01(\x0b\x32).featureform.serving.proto.TrainingDataID\x12/\n\x05model\x18\x02 \x01(\x0b\x32 .featureform.serving.proto.Model\"/\n\x0eTrainingDataID\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\"L\n\x10TrainingDataRows\x12\x38\n\x04rows\x18\x01 \x03(\x0b\x32*.featureform.serving.proto.TrainingDataRow\"v\n\x0fTrainingDataRow\x12\x32\n\x08\x66\x65\x61tures\x18\x01 \x03(\x0b\x32 .featureform.serving.proto.Value\x12/\n\x05label\x18\x02 \x01(\x0b\x32 .featureform.serving.proto.Value\"\xb3\x01\n\x13\x46\x65\x61tureServeRequest\x12\x36\n\x08\x66\x65\x61tures\x18\x01 \x03(\x0b\x32$.featureform.serving.proto.FeatureID\x12\x33\n\x08\x65ntities\x18\x02 \x03(\x0b\x32!.featureform.serving.proto.Entity\x12/\n\x05model\x18\x03 \x01(\x0b\x32 .featureform.serving.proto.Model\"}\n\nFeatureRow\x12\x34\n\x06values\x18\x01 \x03(\x0b\x32 .featureform.serving.proto.ValueB\x02\x18\x01\x12\x39\n\x0bvalue_lists\x18\x02 \x03(\x0b\x32$.featureform.serving.proto.ValueList\"=\n\tValueList\x12\x30\n\x06values\x18\x01 \x03(\x0b\x32 .featureform.serving.proto.Value\"R\n\x18\x42\x61tchFeatureServeRequest\x12\x36\n\x08\x66\x65\x61tures\x18\x01 \x03(\x0b\x32$.featureform.serving.proto.FeatureID\"L\n\x10\x42\x61tchFeatureRows\x12\x38\n\x04rows\x18\x01 \x03(\x0b\x32*.featureform.serving.proto.BatchFeatureRow\"w\n\x0f\x42\x61tchFeatureRow\x12\x30\n\x06\x65ntity\x18\x01 \x01(\x0b\x32 .featureform.serving.proto.Value\x12\x32\n\x08\x66\x65\x61tures\x18\x02 \x03(\x0b\x32 .featureform.serving.proto.Value\"*\n\tFeatureID\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\"9\n\x06\x45ntity\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\x05value\x18\x02 \x01(\tB\x02\x18\x01\x12\x0e\n\x06values\x18\x03 \x03(\t\"\xba\x02\n\x05Value\x12\x13\n\tstr_value\x18\x01 \x01(\tH\x00\x12\x13\n\tint_value\x18\x02 \x01(\x05H\x00\x12\x15\n\x0b\x66loat_value\x18\x03 \x01(\x02H\x00\x12\x16\n\x0c\x64ouble_value\x18\x04 \x01(\x01H\x00\x12\x15\n\x0bint64_value\x18\x05 \x01(\x03H\x00\x12\x15\n\x0bint32_value\x18\x06 \x01(\x05H\x00\x12\x14\n\nbool_value\x18\x07 \x01(\x08H\x00\x12\x1c\n\x12on_demand_function\x18\x08 \x01(\x0cH\x00\x12=\n\x0evector32_value\x18\t \x01(\x0b\x32#.featureform.serving.proto.Vector32H\x00\x12\x16\n\x0cuint32_value\x18\n \x01(\rH\x00\x12\x16\n\x0cuint64_value\x18\x0b \x01(\x04H\x00\x42\x07\n\x05value\")\n\x08SourceID\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\"S\n\x11SourceDataRequest\x12/\n\x02id\x18\x01 \x01(\x0b\x32#.featureform.serving.proto.SourceID\x12\r\n\x05limit\x18\x02 \x01(\x03\"F\n\x13SourceColumnRequest\x12/\n\x02id\x18\x01 \x01(\x0b\x32#.featureform.serving.proto.SourceID\"H\n\x0eSourceDataRows\x12\x36\n\x04rows\x18\x01 \x03(\x0b\x32(.featureform.serving.proto.SourceDataRow\"?\n\rSourceDataRow\x12.\n\x04rows\x18\x01 \x03(\x0b\x32 .featureform.serving.proto.Value\"$\n\x11SourceDataColumns\x12\x0f\n\x07\x63olumns\x18\x01 \x03(\t\"S\n\x1aTrainingDataColumnsRequest\x12\x35\n\x02id\x18\x01 \x01(\x0b\x32).featureform.serving.proto.TrainingDataID\"2\n\x0fTrainingColumns\x12\x10\n\x08\x66\x65\x61tures\x18\x01 \x03(\t\x12\r\n\x05label\x18\x02 \x01(\t\"\x19\n\x08Vector32\x12\r\n\x05value\x18\x01 \x03(\x02\"\x82\x01\n\x0eNearestRequest\x12\x30\n\x02id\x18\x01 \x01(\x0b\x32$.featureform.serving.proto.FeatureID\x12\x33\n\x06vector\x18\x02 \x01(\x0b\x32#.featureform.serving.proto.Vector32\x12\t\n\x01k\x18\x03 \x01(\x05\"#\n\x0fNearestResponse\x12\x10\n\x08\x65ntities\x18\x01 \x03(\t\"@\n\x11ResourceIdRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07variant\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\x05\"$\n\x10ResourceLocation\x12\x10\n\x08location\x18\x01 \x01(\t\"\x9f\x02\n\x15TrainTestSplitRequest\x12\x35\n\x02id\x18\x01 \x01(\x0b\x32).featureform.serving.proto.TrainingDataID\x12/\n\x05model\x18\x02 \x01(\x0b\x32 .featureform.serving.proto.Model\x12\x11\n\ttest_size\x18\x03 \x01(\x02\x12\x12\n\ntrain_size\x18\x04 \x01(\x02\x12\x0f\n\x07shuffle\x18\x05 \x01(\x08\x12\x14\n\x0crandom_state\x18\x06 \x01(\x05\x12<\n\x0crequest_type\x18\x07 \x01(\x0e\x32&.featureform.serving.proto.RequestType\x12\x12\n\nbatch_size\x18\x08 \x01(\x05\"\xd0\x01\n\x1b\x42\x61tchTrainTestSplitResponse\x12<\n\x0crequest_type\x18\x01 \x01(\x0e\x32&.featureform.serving.proto.RequestType\x12\x15\n\riterator_done\x18\x02 \x01(\x08\x12\x15\n\x0binitialized\x18\x03 \x01(\x08H\x00\x12;\n\x04\x64\x61ta\x18\x04 \x01(\x0b\x32+.featureform.serving.proto.TrainingDataRowsH\x00\x42\x08\n\x06result\"\x7f\n\x17\x46\x65\x61tureViewServeRequest\x12\x11\n\tview_name\x18\x01 \x01(\t\x12\x12\n\nentity_ids\x18\x02 \x03(\t\x12=\n\x0f\x66ilter_features\x18\x03 \x03(\x0b\x32$.featureform.serving.proto.FeatureID\"A\n\rFeatureColumn\x12\x16\n\x0e\x66\x65\x61ture_column\x18\x01 \x01(\t\x12\x18\n\x10timestamp_column\x18\x02 \x01(\t\"m\n\x11\x46\x65\x61tureViewSchema\x12\x15\n\rentity_column\x18\x01 \x01(\t\x12\x41\n\x0f\x66\x65\x61ture_columns\x18\x02 \x03(\x0b\x32(.featureform.serving.proto.FeatureColumn\"\x8a\x01\n\x0f\x46\x65\x61tureViewRows\x12<\n\x06schema\x18\x01 \x01(\x0b\x32,.featureform.serving.proto.FeatureViewSchema\x12\x39\n\x0bvalue_lists\x18\x02 \x03(\x0b\x32$.featureform.serving.proto.ValueList*5\n\x0bRequestType\x12\x0e\n\nINITIALIZE\x10\x00\x12\x0c\n\x08TRAINING\x10\x01\x12\x08\n\x04TEST\x10\x02\x32\x87\t\n\x07\x46\x65\x61ture\x12o\n\x0cTrainingData\x12..featureform.serving.proto.TrainingDataRequest\x1a+.featureform.serving.proto.TrainingDataRows\"\x00\x30\x01\x12\x80\x01\n\x0eTrainTestSplit\x12\x30.featureform.serving.proto.TrainTestSplitRequest\x1a\x36.featureform.serving.proto.BatchTrainTestSplitResponse\"\x00(\x01\x30\x01\x12z\n\x13TrainingDataColumns\x12\x35.featureform.serving.proto.TrainingDataColumnsRequest\x1a*.featureform.serving.proto.TrainingColumns\"\x00\x12g\n\x0c\x46\x65\x61tureServe\x12..featureform.serving.proto.FeatureServeRequest\x1a%.featureform.serving.proto.FeatureRow\"\x00\x12i\n\nSourceData\x12,.featureform.serving.proto.SourceDataRequest\x1a).featureform.serving.proto.SourceDataRows\"\x00\x30\x01\x12o\n\rSourceColumns\x12..featureform.serving.proto.SourceColumnRequest\x1a,.featureform.serving.proto.SourceDataColumns\"\x00\x12\x62\n\x07Nearest\x12).featureform.serving.proto.NearestRequest\x1a*.featureform.serving.proto.NearestResponse\"\x00\x12y\n\x11\x42\x61tchFeatureServe\x12\x33.featureform.serving.proto.BatchFeatureServeRequest\x1a+.featureform.serving.proto.BatchFeatureRows\"\x00\x30\x01\x12r\n\x13GetResourceLocation\x12,.featureform.serving.proto.ResourceIdRequest\x1a+.featureform.serving.proto.ResourceLocation\"\x00\x12t\n\x10ServeFeatureView\x12\x32.featureform.serving.proto.FeatureViewServeRequest\x1a*.featureform.serving.proto.FeatureViewRows\"\x00\x42\x1eZ\x1cgithub.com/featureform/protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'featureform.proto.serving_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z\034github.com/featureform/proto'
  _globals['_FEATUREROW'].fields_by_name['values']._options = None
  _globals['_FEATUREROW'].fields_by_name['values']._serialized_options = b'\030\001'
  _globals['_ENTITY'].fields_by_name['value']._options = None
  _globals['_ENTITY'].fields_by_name['value']._serialized_options = b'\030\001'
  _globals['_REQUESTTYPE']._serialized_start=3298
  _globals['_REQUESTTYPE']._serialized_end=3351
  _globals['_MODEL']._serialized_start=62
  _globals['_MODEL']._serialized_end=83
  _globals['_TRAININGDATAREQUEST']._serialized_start=85
  _globals['_TRAININGDATAREQUEST']._serialized_end=210
  _globals['_TRAININGDATAID']._serialized_start=212
  _globals['_TRAININGDATAID']._serialized_end=259
  _globals['_TRAININGDATAROWS']._serialized_start=261
  _globals['_TRAININGDATAROWS']._serialized_end=337
  _globals['_TRAININGDATAROW']._serialized_start=339
  _globals['_TRAININGDATAROW']._serialized_end=457
  _globals['_FEATURESERVEREQUEST']._serialized_start=460
  _globals['_FEATURESERVEREQUEST']._serialized_end=639
  _globals['_FEATUREROW']._serialized_start=641
  _globals['_FEATUREROW']._serialized_end=766
  _globals['_VALUELIST']._serialized_start=768
  _globals['_VALUELIST']._serialized_end=829
  _globals['_BATCHFEATURESERVEREQUEST']._serialized_start=831
  _globals['_BATCHFEATURESERVEREQUEST']._serialized_end=913
  _globals['_BATCHFEATUREROWS']._serialized_start=915
  _globals['_BATCHFEATUREROWS']._serialized_end=991
  _globals['_BATCHFEATUREROW']._serialized_start=993
  _globals['_BATCHFEATUREROW']._serialized_end=1112
  _globals['_FEATUREID']._serialized_start=1114
  _globals['_FEATUREID']._serialized_end=1156
  _globals['_ENTITY']._serialized_start=1158
  _globals['_ENTITY']._serialized_end=1215
  _globals['_VALUE']._serialized_start=1218
  _globals['_VALUE']._serialized_end=1532
  _globals['_SOURCEID']._serialized_start=1534
  _globals['_SOURCEID']._serialized_end=1575
  _globals['_SOURCEDATAREQUEST']._serialized_start=1577
  _globals['_SOURCEDATAREQUEST']._serialized_end=1660
  _globals['_SOURCECOLUMNREQUEST']._serialized_start=1662
  _globals['_SOURCECOLUMNREQUEST']._serialized_end=1732
  _globals['_SOURCEDATAROWS']._serialized_start=1734
  _globals['_SOURCEDATAROWS']._serialized_end=1806
  _globals['_SOURCEDATAROW']._serialized_start=1808
  _globals['_SOURCEDATAROW']._serialized_end=1871
  _globals['_SOURCEDATACOLUMNS']._serialized_start=1873
  _globals['_SOURCEDATACOLUMNS']._serialized_end=1909
  _globals['_TRAININGDATACOLUMNSREQUEST']._serialized_start=1911
  _globals['_TRAININGDATACOLUMNSREQUEST']._serialized_end=1994
  _globals['_TRAININGCOLUMNS']._serialized_start=1996
  _globals['_TRAININGCOLUMNS']._serialized_end=2046
  _globals['_VECTOR32']._serialized_start=2048
  _globals['_VECTOR32']._serialized_end=2073
  _globals['_NEARESTREQUEST']._serialized_start=2076
  _globals['_NEARESTREQUEST']._serialized_end=2206
  _globals['_NEARESTRESPONSE']._serialized_start=2208
  _globals['_NEARESTRESPONSE']._serialized_end=2243
  _globals['_RESOURCEIDREQUEST']._serialized_start=2245
  _globals['_RESOURCEIDREQUEST']._serialized_end=2309
  _globals['_RESOURCELOCATION']._serialized_start=2311
  _globals['_RESOURCELOCATION']._serialized_end=2347
  _globals['_TRAINTESTSPLITREQUEST']._serialized_start=2350
  _globals['_TRAINTESTSPLITREQUEST']._serialized_end=2637
  _globals['_BATCHTRAINTESTSPLITRESPONSE']._serialized_start=2640
  _globals['_BATCHTRAINTESTSPLITRESPONSE']._serialized_end=2848
  _globals['_FEATUREVIEWSERVEREQUEST']._serialized_start=2850
  _globals['_FEATUREVIEWSERVEREQUEST']._serialized_end=2977
  _globals['_FEATURECOLUMN']._serialized_start=2979
  _globals['_FEATURECOLUMN']._serialized_end=3044
  _globals['_FEATUREVIEWSCHEMA']._serialized_start=3046
  _globals['_FEATUREVIEWSCHEMA']._serialized_end=3155
  _globals['_FEATUREVIEWROWS']._serialized_start=3158
  _globals['_FEATUREVIEWROWS']._serialized_end=3296
  _globals['_FEATURE']._serialized_start=3354
  _globals['_FEATURE']._serialized_end=4513
# @@protoc_insertion_point(module_scope)



from dataclasses import dataclass
from enum import Enum
from fnmatch import fnmatch
from os import path

from dataclasses import dataclass
from typeguard import typechecked

from featureform.proto import metadata_pb2 as pb


class ScalarType(str, Enum):
    """
    ScalarType is an enum of all the scalar types supported by Featureform.

    Attributes:
        NIL: An empty string representing no specified type.
        INT: A string representing an integer type.
        INT32: A string representing a 32-bit integer type.
        INT64: A string representing a 64-bit integer type.
        FLOAT32: A string representing a 32-bit float type.
        FLOAT64: A string representing a 64-bit float type.
        STRING: A string representing a string type.
        BOOL: A string representing a boolean type.
        DATETIME: A string representing a datetime type.
    """

    NIL = ""
    INT = "int"
    INT32 = "int32"
    INT64 = "int64"
    FLOAT32 = "float32"
    FLOAT64 = "float64"
    STRING = "string"
    BOOL = "bool"
    DATETIME = "datetime"
    TIMESTAMP = "timestamp"

    @classmethod
    def has_value(cls, value):
        try:
            cls(value)
            return True
        except ValueError:
            return False

    @classmethod
    def get_values(cls):
        return [e.value for e in cls]

    def to_proto(self):
        proto_enum = self.to_proto_enum()
        return pb.ValueType(scalar=proto_enum)

    def to_proto_enum(self):
        mapping = {
            ScalarType.NIL: pb.ScalarType.NULL,
            ScalarType.INT: pb.ScalarType.INT,
            ScalarType.INT32: pb.ScalarType.INT32,
            ScalarType.INT64: pb.ScalarType.INT64,
            ScalarType.FLOAT32: pb.ScalarType.FLOAT32,
            ScalarType.FLOAT64: pb.ScalarType.FLOAT64,
            ScalarType.STRING: pb.ScalarType.STRING,
            ScalarType.BOOL: pb.ScalarType.BOOL,
            ScalarType.DATETIME: pb.ScalarType.DATETIME,
            ScalarType.TIMESTAMP: pb.ScalarType.TIMESTAMP,
        }
        return mapping[self]

    @classmethod
    def from_proto(cls, proto_val):
        mapping = {
            pb.ScalarType.NULL: ScalarType.NIL,
            pb.ScalarType.INT: ScalarType.INT,
            pb.ScalarType.INT32: ScalarType.INT32,
            pb.ScalarType.INT64: ScalarType.INT64,
            pb.ScalarType.FLOAT32: ScalarType.FLOAT32,
            pb.ScalarType.FLOAT64: ScalarType.FLOAT64,
            pb.ScalarType.STRING: ScalarType.STRING,
            pb.ScalarType.BOOL: ScalarType.BOOL,
            pb.ScalarType.DATETIME: ScalarType.DATETIME,
            pb.ScalarType.TIMESTAMP: ScalarType.TIMESTAMP,
        }
        return mapping[proto_val]


class ResourceStatus(str, Enum):
    """
    ResourceStatus is an enumeration representing the possible states that a
    resource may occupy within an application.

    Each status is represented as a string, which provides a human-readable
    representation for each of the stages in the lifecycle of a resource.

    Attributes:
        NO_STATUS (str): The state of a resource that cannot have another status.
        CREATED (str): The state after a resource has been successfully created.
        PENDING (str): The state indicating that the resource is in the process of being prepared, but is not yet ready.
        READY (str): The state indicating that the resource has been successfully prepared and is now ready for use.
        FAILED (str): The state indicating that an error occurred during the creation or preparation of the resource.
    """

    NO_STATUS = "NO_STATUS"
    CREATED = "CREATED"
    PENDING = "PENDING"
    READY = "READY"
    FAILED = "FAILED"
    RUNNING = "RUNNING"

    @staticmethod
    def from_proto(proto):
        return proto.Status._enum_type.values[proto.status].name


class ComputationMode(Enum):
    PRECOMPUTED = "PRECOMPUTED"
    CLIENT_COMPUTED = "CLIENT_COMPUTED"
    STREAMING = "STREAMING"

    def __eq__(self, other: str) -> bool:
        return self.value == other

    def proto(self) -> int:
        if self == ComputationMode.PRECOMPUTED:
            return pb.ComputationMode.PRECOMPUTED
        elif self == ComputationMode.CLIENT_COMPUTED:
            return pb.ComputationMode.CLIENT_COMPUTED
        elif self == ComputationMode.STREAMING:
            return pb.ComputationMode.STREAMING
        else:
            raise ValueError(f"Unknown computation mode: {self}")


@typechecked
@dataclass
class OperationType(Enum):
    GET = 0
    CREATE = 1


@typechecked
@dataclass
class SourceType(str, Enum):
    PRIMARY_SOURCE = "PRIMARY"
    DIRECTORY = "DIRECTORY"
    DF_TRANSFORMATION = "DF"
    SQL_TRANSFORMATION = "SQL"


class FilePrefix(Enum):
    S3 = ("s3://", "s3a://")
    S3A = ("s3a://",)
    HDFS = ("hdfs://",)
    GCS = ("gs://",)
    AZURE = ("abfss://",)

    def __init__(self, *valid_prefixes):
        self.prefixes = valid_prefixes

    @property
    def value(self):
        return self.prefixes[0]

    def validate_file_scheme(self, file_path: str) -> (bool, str):
        if not any(file_path.startswith(prefix) for prefix in self.prefixes):
            raise Exception(
                f"File path '{file_path}' must be a full path. Must start with '{self.prefixes}'"
            )

    @staticmethod
    def validate(store_type: str, file_path: str):
        try:
            prefix = FilePrefix[store_type]
            prefix.validate_file_scheme(file_path)
        except KeyError:
            raise Exception(f"Invalid store type: {store_type}")


class FileFormat(str, Enum):
    CSV = "csv"
    PARQUET = "parquet"

    @classmethod
    def is_supported(cls, file_path: str) -> bool:
        file_name = path.basename(file_path)

        for file_format in cls:
            if fnmatch(file_name, f"*.{file_format.value}"):
                return True

        return False

    @classmethod
    def get_format(cls, file_path: str, default: str = "") -> str:
        file_name = path.basename(file_path)

        for file_format in cls:
            if fnmatch(file_name, f"*.{file_format.value}"):
                return file_format.value

        if default != "":
            return default
        raise ValueError(f"File format not supported: {file_name}")

    @classmethod
    def supported_formats(cls) -> str:
        return ", ".join([file_format.value for file_format in cls])


@typechecked
class DataResourceType(Enum):
    # ResourceType is an enumeration representing the possible types of
    # resources that may be registered with Featureform. Each value is based
    # on OfflineResourceType in providers/offline.go

    NO_TYPE = 0
    LABEL = 1
    FEATURE = 2
    TRAINING_SET = 3
    PRIMARY = 4
    TRANSFORMATION = 5
    FEATURE_MATERIALIZATION = 6


class ResourceType(Enum):
    NO_TYPE = 0
    USER = 1
    PROVIDER = 2
    SOURCE_VARIANT = 3
    ENTITY = 4
    FEATURE_VARIANT = 5
    ONDEMAND_FEATURE = 6
    LABEL_VARIANT = 7
    TRAININGSET_VARIANT = 8
    SCHEDULE = 9
    MODEL = 10
    TRANSFORMATION = 11
    FEATURE_VIEW = 12
    STREAM_CHANNEL = 13

    @classmethod
    def _get_proto_map(cls):
        return {
            cls.NO_TYPE: None,
            cls.USER: pb.ResourceType.USER,
            cls.PROVIDER: pb.ResourceType.PROVIDER,
            cls.SOURCE_VARIANT: pb.ResourceType.SOURCE_VARIANT,
            cls.ENTITY: pb.ResourceType.ENTITY,
            cls.FEATURE_VARIANT: pb.ResourceType.FEATURE_VARIANT,
            cls.ONDEMAND_FEATURE: pb.ResourceType.FEATURE_VARIANT,
            cls.LABEL_VARIANT: pb.ResourceType.LABEL_VARIANT,
            cls.TRAININGSET_VARIANT: pb.ResourceType.TRAINING_SET_VARIANT,
            cls.SCHEDULE: None,
            cls.MODEL: pb.ResourceType.MODEL,
            cls.TRANSFORMATION: pb.ResourceType.SOURCE_VARIANT,
            cls.FEATURE_VIEW: pb.ResourceType.FEATURE_VIEW,
            cls.STREAM_CHANNEL: pb.ResourceType.STREAM_CHANNEL,
        }

    def to_proto(self):
        return self._get_proto_map()[self]

    def to_string(self) -> str:
        return self.name.replace("_", " ").title()

    @classmethod
    def is_deletable(cls, resource_type):
        return resource_type in [
            # cls.USER,
            cls.PROVIDER,
            cls.SOURCE_VARIANT,
            # cls.ENTITY,
            cls.FEATURE_VARIANT,
            # cls.ONDEMAND_FEATURE,
            cls.LABEL_VARIANT,
            cls.TRAININGSET_VARIANT,
            # cls.SCHEDULE,
            # cls.MODEL,
            cls.TRANSFORMATION,
            cls.FEATURE_VIEW,
            cls.STREAM_CHANNEL,
        ]


@typechecked
@dataclass
class TableFormat(str, Enum):
    ICEBERG = "iceberg"
    DELTA = "delta"

    @classmethod
    def get_format(cls, format: str):
        try:
            return cls(format)
        except ValueError:
            raise ValueError(f"Table format not supported: {format}")

    @classmethod
    def is_supported(cls, table_format: str) -> bool:
        return table_format in [table_format.value for table_format in cls]

    @classmethod
    def supported_formats(cls) -> str:
        return ", ".join([table_format.value for table_format in cls])


class LocationType(str, Enum):
    TABLE = "table"
    FILESTORE = "filestore"
    CATALOG = "catalog"
    KAFKA = "kafka"


class RefreshMode(Enum):
    AUTO = pb.RefreshMode.REFRESH_MODE_AUTO
    FULL = pb.RefreshMode.REFRESH_MODE_FULL
    INCREMENTAL = pb.RefreshMode.REFRESH_MODE_INCREMENTAL

    @classmethod
    def from_proto(cls, proto_value):
        try:
            return cls(proto_value)
        except ValueError:
            return None

    def to_proto(self):
        return self.value

    def to_string(self):
        return self.name

    @classmethod
    def from_string(cls, value):
        try:
            return cls[value.upper()]
        except KeyError:
            raise ValueError(f"Refresh Mode value not supported: {value}")
        except AttributeError:
            raise ValueError(f"Refresh Mode value required: received {value}")


class Initialize(Enum):
    ON_CREATE = pb.Initialize.INITIALIZE_ON_CREATE
    ON_SCHEDULE = pb.Initialize.INITIALIZE_ON_SCHEDULE

    @classmethod
    def from_proto(cls, proto_value):
        try:
            return cls(proto_value)
        except ValueError:
            return None

    def to_proto(self):
        return self.value

    def to_string(self):
        return self.name

    @classmethod
    def from_string(cls, value):
        try:
            return cls[value.upper()]
        except KeyError:
            raise ValueError(f"Initialize value not supported: {value}")
        except AttributeError:
            raise ValueError(f"Initialize value required: received {value}")


class SnowflakeSessionParamKey(Enum):
    ABORT_DETACHED_QUERY = "abort_detached_query"
    AUTOCOMMIT = "autocommit"
    BINARY_INPUT_FORMAT = "binary_input_format"
    BINARY_OUTPUT_FORMAT = "binary_output_format"
    DATE_INPUT_FORMAT = "date_input_format"
    DATE_OUTPUT_FORMAT = "date_output_format"
    ERROR_ON_NONDETERMINISTIC_MERGE = "error_on_nondeterministic_merge"
    ERROR_ON_NONDETERMINISTIC_UPDATE = "error_on_nondeterministic_update"
    GEOGRAPHY_OUTPUT_FORMAT = "geography_output_format"
    HYBRID_TABLE_LOCK_TIMEOUT = "hybrid_table_lock_timeout"
    JSON_INDENT = "json_indent"
    LOG_LEVEL = "log_level"
    LOCK_TIMEOUT = "lock_timeout"
    QUERY_TAG = "query_tag"
    ROWS_PER_RESULTSET = "rows_per_resultset"
    S3_STAGE_VPCE_DNS_NAME = "s3_stage_vpce_dns_name"
    SEARCH_PATH = "search_path"
    SIMULATED_DATA_SHARING_CONSUMER = "simulated_data_sharing_consumer"
    STATEMENT_TIMEOUT_IN_SECONDS = "statement_timeout_in_seconds"
    STRICT_JSON_OUTPUT = "strict_json_output"
    TIMESTAMP_DAY_IS_ALWAYS_24H = "timestamp_day_is_always_24h"
    TIMESTAMP_INPUT_FORMAT = "timestamp_input_format"
    TIMESTAMP_LTZ_OUTPUT_FORMAT = "timestamp_ltz_output_format"
    TIMESTAMP_NTZ_OUTPUT_FORMAT = "timestamp_ntz_output_format"
    TIMESTAMP_OUTPUT_FORMAT = "timestamp_output_format"
    TIMESTAMP_TYPE_MAPPING = "timestamp_type_mapping"
    TIMESTAMP_TZ_OUTPUT_FORMAT = "timestamp_tz_output_format"
    TIMEZONE = "timezone"
    TIME_INPUT_FORMAT = "time_input_format"
    TIME_OUTPUT_FORMAT = "time_output_format"
    TRACE_LEVEL = "trace_level"
    TRANSACTION_DEFAULT_ISOLATION_LEVEL = "transaction_default_isolation_level"
    TWO_DIGIT_CENTURY_START = "two_digit_century_start"
    UNSUPPORTED_DDL_ACTION = "unsupported_ddl_action"
    USE_CACHED_RESULT = "use_cached_result"
    WEEK_OF_YEAR_POLICY = "week_of_year_policy"
    WEEK_START = "week_start"

    @classmethod
    def validate_key(cls, key: str) -> bool:
        return key.lower() in cls._value2member_map_


class TrainingSetType(Enum):
    # Dynamic training sets are updated when new rows are added to upstream tables;
    # mechanism of update will depend on the offline provider (e.g. Snowflake uses dynamic tables).
    DYNAMIC = pb.TrainingSetType.TRAINING_SET_TYPE_DYNAMIC
    # Static training sets are not updated when new rows are added to upstream tables; they are
    # created as regular tables and are not updated.
    STATIC = pb.TrainingSetType.TRAINING_SET_TYPE_STATIC
    # View training sets are created as views and are only evaluated when queried by a user
    # (e.g. when fetching the training set for model training).
    VIEW = pb.TrainingSetType.TRAINING_SET_TYPE_VIEW

    @classmethod
    def from_proto(cls, proto_value):
        try:
            return cls(proto_value)
        except ValueError:
            return None

    def to_proto(self):
        return self.value

    def to_string(self):
        return self.name

    @classmethod
    def from_string(cls, value):
        try:
            return cls[value.upper()]
        except KeyError:
            raise ValueError(f"Training Set Type value not supported: {value}")
        except AttributeError:
            raise ValueError(f"Training Set Type value required: received {value}")



import ast
import inspect
import re
import textwrap
from typing import Callable

import astor


def add_variant_to_name(text, replacement):
    return re.sub(r"\{\{\s*(\w+)\s*\}\}", r"{{ \1." + replacement + r" }}", text)


def canonicalize_function_definition(fn: Callable) -> str:
    """
    Takes a function definition and returns a string with comments and extra whitespace removed,
    making it easier to compare the function definitions for equality.

    The function parses the input function using the `astor` module, removes comments and docstrings,
    and then converts the Abstract Syntax Tree (AST) back into source code.

    Example:
    canonicalize_function_definition('''
    def foo(bar: int) -> int:
        # This is a comment
        return bar
    ''')
    returns:
    'def foo(bar: int) -> int: return bar'

    Args:
        fn (Callable): The function whose definition will be canonicalized.

    Raises:
        TypeError: If `fn` is None or not a valid function.
        ValueError: If the function's source code can't be parsed.

    Returns:
        str: A string representing the cleaned function definition without comments or docstrings.
    """

    if fn is None:
        raise TypeError("Function is required")

    # get sources from the function
    text = inspect.getsource(fn)

    dedented_code = textwrap.dedent(text)

    function_without_decorator = _remove_decorator(dedented_code)

    try:
        tree = ast.parse(function_without_decorator)
    except (SyntaxError, ValueError) as e:
        raise ValueError(f"Error parsing source code: {e}") from e

    class NormalizeAst(ast.NodeTransformer):
        def remove_docstring(self, node):
            """Removes comments if present as the first element of a node body."""
            while (
                node.body
                and isinstance(node.body[0], ast.Expr)
                and isinstance(node.body[0].value, ast.Constant)
            ):
                node.body.pop(0)
            return node

        def visit_FunctionDef(self, node):
            # Remove function docstring
            self.remove_docstring(node)
            return self.generic_visit(node)

        def visit_ClassDef(self, node):
            # Remove class docstring
            self.remove_docstring(node)
            return self.generic_visit(node)

        def visit_Module(self, node):
            # Remove module docstring
            self.remove_docstring(node)
            return self.generic_visit(node)

    normalizer = NormalizeAst()
    normalized_tree = normalizer.visit(tree)
    ast.fix_missing_locations(normalized_tree)

    normalized_code = astor.to_source(normalized_tree).strip()

    return normalized_code


def _remove_decorator(fn_text: str):
    module_node = ast.parse(fn_text)
    func_def_node = module_node.body[0]
    func_def_node.decorator_list = []
    function_without_decorator = astor.to_source(func_def_node)
    return function_without_decorator



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: featureform/proto/metadata.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n featureform/proto/metadata.proto\x12\"featureform.serving.metadata.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x19google/protobuf/any.proto\"k\n\x0eOktaAuthConfig\x12\x0e\n\x06\x64omain\x18\x01 \x01(\t\x12\x1f\n\x17\x61uthorization_server_id\x18\x02 \x01(\t\x12\x11\n\tclient_id\x18\x03 \x01(\t\x12\x15\n\rapi_client_id\x18\x04 \x01(\t\"\x17\n\x15PassThroughAuthConfig\"\x10\n\x0e\x43IDRAuthConfig\"\xf1\x01\n\nAuthConfig\x12\x42\n\x04okta\x18\x01 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.OktaAuthConfigH\x00\x12Q\n\x0cpass_through\x18\x02 \x01(\x0b\x32\x39.featureform.serving.metadata.proto.PassThroughAuthConfigH\x00\x12\x42\n\x04\x63idr\x18\x03 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.CIDRAuthConfigH\x00\x42\x08\n\x06\x63onfig\"\x14\n\x04Name\x12\x0c\n\x04name\x18\x01 \x01(\t\"Y\n\x0bNameRequest\x12\x36\n\x04name\x18\x01 \x01(\x0b\x32(.featureform.serving.metadata.proto.Name\x12\x12\n\nrequest_id\x18\x02 \x01(\t\"S\n\x0b\x45rrorStatus\x12\x0c\n\x04\x63ode\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\x12%\n\x07\x64\x65tails\x18\x03 \x03(\x0b\x32\x14.google.protobuf.Any\"\x9f\x02\n\x0eResourceStatus\x12I\n\x06status\x18\x01 \x01(\x0e\x32\x39.featureform.serving.metadata.proto.ResourceStatus.Status\x12\x15\n\rerror_message\x18\x02 \x01(\t\x12\x45\n\x0c\x65rror_status\x18\x03 \x01(\x0b\x32/.featureform.serving.metadata.proto.ErrorStatus\"d\n\x06Status\x12\r\n\tNO_STATUS\x10\x00\x12\x0b\n\x07\x43REATED\x10\x01\x12\x0b\n\x07PENDING\x10\x02\x12\t\n\x05READY\x10\x03\x12\n\n\x06\x46\x41ILED\x10\x04\x12\x0b\n\x07RUNNING\x10\x05\x12\r\n\tCANCELLED\x10\x06\"\x98\x01\n\nResourceID\x12\x41\n\x08resource\x18\x01 \x01(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12G\n\rresource_type\x18\x02 \x01(\x0e\x32\x30.featureform.serving.metadata.proto.ResourceType\"\x9b\x01\n\x10SetStatusRequest\x12\x43\n\x0bresource_id\x18\x01 \x01(\x0b\x32..featureform.serving.metadata.proto.ResourceID\x12\x42\n\x06status\x18\x02 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\"n\n\x15ScheduleChangeRequest\x12\x43\n\x0bresource_id\x18\x01 \x01(\x0b\x32..featureform.serving.metadata.proto.ResourceID\x12\x10\n\x08schedule\x18\x02 \x01(\t\",\n\x0bNameVariant\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07variant\x18\x02 \x01(\t\"o\n\x12NameVariantRequest\x12\x45\n\x0cname_variant\x18\x01 \x01(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x12\n\nrequest_id\x18\x02 \x01(\t\"\x07\n\x05\x45mpty\"!\n\x0bListRequest\x12\x12\n\nrequest_id\x18\x01 \x01(\t\"\x86\x01\n\x07\x46\x65\x61ture\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x42\n\x06status\x18\x02 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12\x17\n\x0f\x64\x65\x66\x61ult_variant\x18\x03 \x01(\t\x12\x10\n\x08variants\x18\x04 \x03(\t\"4\n\x07\x43olumns\x12\x0e\n\x06\x65ntity\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\x12\n\n\x02ts\x18\x03 \x01(\t\"\x1f\n\x0ePythonFunction\x12\r\n\x05query\x18\x01 \x01(\x0c\"\"\n\x06Stream\x12\x18\n\x10offline_provider\x18\x01 \x01(\t\"\xa5\x03\n\x0fResourceVariant\x12M\n\x0f\x66\x65\x61ture_variant\x18\x01 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.FeatureVariantH\x00\x12I\n\rlabel_variant\x18\x02 \x01(\x0b\x32\x30.featureform.serving.metadata.proto.LabelVariantH\x00\x12V\n\x14training_set_variant\x18\x03 \x01(\x0b\x32\x36.featureform.serving.metadata.proto.TrainingSetVariantH\x00\x12K\n\x0esource_variant\x18\x04 \x01(\x0b\x32\x31.featureform.serving.metadata.proto.SourceVariantH\x00\x12G\n\x0c\x66\x65\x61ture_view\x18\x05 \x01(\x0b\x32/.featureform.serving.metadata.proto.FeatureViewH\x00\x42\n\n\x08resource\"p\n\x14GetEquivalentRequest\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12\x44\n\x07variant\x18\x02 \x01(\x0b\x32\x33.featureform.serving.metadata.proto.ResourceVariant\"g\n\nRunRequest\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12\x45\n\x08variants\x18\x02 \x03(\x0b\x32\x33.featureform.serving.metadata.proto.ResourceVariant\"\xb0\n\n\x0e\x46\x65\x61tureVariant\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07variant\x18\x02 \x01(\t\x12?\n\x06source\x18\x03 \x01(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x0e\n\x06\x65ntity\x18\x05 \x01(\t\x12+\n\x07\x63reated\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\r\n\x05owner\x18\x07 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x08 \x01(\t\x12\x10\n\x08provider\x18\t \x01(\t\x12\x42\n\x06status\x18\n \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12\x45\n\x0ctrainingsets\x18\x0b \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12>\n\x07\x63olumns\x18\x0c \x01(\x0b\x32+.featureform.serving.metadata.proto.ColumnsH\x00\x12\x46\n\x08\x66unction\x18\x11 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.PythonFunctionH\x00\x12<\n\x06stream\x18\x15 \x01(\x0b\x32*.featureform.serving.metadata.proto.StreamH\x00\x12\x30\n\x0clast_updated\x18\r \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x10\n\x08schedule\x18\x0e \x01(\t\x12\x36\n\x04tags\x18\x0f \x01(\x0b\x32(.featureform.serving.metadata.proto.Tags\x12\x42\n\nproperties\x18\x10 \x01(\x0b\x32..featureform.serving.metadata.proto.Properties\x12\x41\n\x04mode\x18\x12 \x01(\x0e\x32\x33.featureform.serving.metadata.proto.ComputationMode\x12\x13\n\x07task_id\x18\x17 \x01(\tB\x02\x18\x01\x12\x14\n\x0ctask_id_list\x18\x18 \x03(\t\x12T\n\x15\x61\x64\x64itional_parameters\x18\x16 \x01(\x0b\x32\x35.featureform.serving.metadata.proto.FeatureParameters\x12;\n\x04type\x18\x19 \x01(\x0b\x32-.featureform.serving.metadata.proto.ValueType\x12^\n\x19resource_snowflake_config\x18\x1a \x01(\x0b\x32;.featureform.serving.metadata.proto.ResourceSnowflakeConfig\x12\x16\n\nis_deleted\x18\x1b \x01(\x08\x42\x02\x18\x01\x12/\n\x07\x64\x65leted\x18\x1c \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x02\x18\x01\x12\x1e\n\x16offline_store_provider\x18\x1d \x01(\t\x12M\n\x17offline_store_locations\x18\x1e \x03(\x0b\x32,.featureform.serving.metadata.proto.Location\x12\x15\n\rfeature_views\x18\x1f \x03(\tB\n\n\x08location\"x\n\x15\x46\x65\x61tureVariantRequest\x12K\n\x0f\x66\x65\x61ture_variant\x18\x01 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.FeatureVariant\x12\x12\n\nrequest_id\x18\x02 \x01(\t\"\x97\x01\n\tValueType\x12@\n\x06scalar\x18\x01 \x01(\x0e\x32..featureform.serving.metadata.proto.ScalarTypeH\x00\x12@\n\x06vector\x18\x02 \x01(\x0b\x32..featureform.serving.metadata.proto.VectorTypeH\x00\x42\x06\n\x04Type\"u\n\nVectorType\x12>\n\x06scalar\x18\x01 \x01(\x0e\x32..featureform.serving.metadata.proto.ScalarType\x12\x11\n\tdimension\x18\x02 \x01(\x05\x12\x14\n\x0cis_embedding\x18\x03 \x01(\x08\"\xcf\x01\n\x11\x46\x65\x61tureParameters\x12W\n\x0bprecomputed\x18\x01 \x01(\x0b\x32@.featureform.serving.metadata.proto.PrecomputedFeatureParametersH\x00\x12Q\n\x08ondemand\x18\x02 \x01(\x0b\x32=.featureform.serving.metadata.proto.OndemandFeatureParametersH\x00\x42\x0e\n\x0c\x66\x65\x61ture_type\"\x1e\n\x1cPrecomputedFeatureParameters\"/\n\x19OndemandFeatureParameters\x12\x12\n\ndefinition\x18\x01 \x01(\t\"d\n\nFeatureLag\x12\x0f\n\x07\x66\x65\x61ture\x18\x01 \x01(\t\x12\x0f\n\x07variant\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12&\n\x03lag\x18\x04 \x01(\x0b\x32\x19.google.protobuf.Duration\"\x84\x01\n\x05Label\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x42\n\x06status\x18\x02 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12\x17\n\x0f\x64\x65\x66\x61ult_variant\x18\x03 \x01(\t\x12\x10\n\x08variants\x18\x04 \x03(\t\"\xd2\x07\n\x0cLabelVariant\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07variant\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12?\n\x06source\x18\x05 \x01(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x0e\n\x06\x65ntity\x18\x06 \x01(\t\x12+\n\x07\x63reated\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\r\n\x05owner\x18\x08 \x01(\t\x12\x10\n\x08provider\x18\t \x01(\t\x12\x42\n\x06status\x18\n \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12\x45\n\x0ctrainingsets\x18\x0b \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12>\n\x07\x63olumns\x18\x0c \x01(\x0b\x32+.featureform.serving.metadata.proto.ColumnsH\x00\x12<\n\x06stream\x18\x10 \x01(\x0b\x32*.featureform.serving.metadata.proto.StreamH\x00\x12M\n\x0f\x65ntity_mappings\x18\x16 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.EntityMappingsH\x00\x12\x36\n\x04tags\x18\r \x01(\x0b\x32(.featureform.serving.metadata.proto.Tags\x12\x42\n\nproperties\x18\x0e \x01(\x0b\x32..featureform.serving.metadata.proto.Properties\x12\x13\n\x07task_id\x18\x0f \x01(\tB\x02\x18\x01\x12;\n\x04type\x18\x12 \x01(\x0b\x32-.featureform.serving.metadata.proto.ValueType\x12\x14\n\x0ctask_id_list\x18\x11 \x03(\t\x12^\n\x19resource_snowflake_config\x18\x13 \x01(\x0b\x32;.featureform.serving.metadata.proto.ResourceSnowflakeConfig\x12\x16\n\nis_deleted\x18\x14 \x01(\x08\x42\x02\x18\x01\x12/\n\x07\x64\x65leted\x18\x15 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x02\x18\x01\x42\n\n\x08location\"\x85\x01\n\x0e\x45ntityMappings\x12\x43\n\x08mappings\x18\x01 \x03(\x0b\x32\x31.featureform.serving.metadata.proto.EntityMapping\x12\x14\n\x0cvalue_column\x18\x02 \x01(\t\x12\x18\n\x10timestamp_column\x18\x03 \x01(\t\"4\n\rEntityMapping\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x15\n\rentity_column\x18\x02 \x01(\t\"r\n\x13LabelVariantRequest\x12G\n\rlabel_variant\x18\x01 \x01(\x0b\x32\x30.featureform.serving.metadata.proto.LabelVariant\x12\x12\n\nrequest_id\x18\x02 \x01(\t\"\x8c\x05\n\x08Provider\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\t\x12\x10\n\x08software\x18\x04 \x01(\t\x12\x0c\n\x04team\x18\x05 \x01(\t\x12\x19\n\x11serialized_config\x18\x06 \x01(\x0c\x12\x42\n\x06status\x18\x07 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12@\n\x07sources\x18\x08 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x41\n\x08\x66\x65\x61tures\x18\t \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x45\n\x0ctrainingsets\x18\n \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12?\n\x06labels\x18\x0b \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x36\n\x04tags\x18\x0c \x01(\x0b\x32(.featureform.serving.metadata.proto.Tags\x12\x42\n\nproperties\x18\r \x01(\x0b\x32..featureform.serving.metadata.proto.Properties\x12\x16\n\nis_deleted\x18\x0e \x01(\x08\x42\x02\x18\x01\x12/\n\x07\x64\x65leted\x18\x0f \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x02\x18\x01\"e\n\x0fProviderRequest\x12>\n\x08provider\x18\x01 \x01(\x0b\x32,.featureform.serving.metadata.proto.Provider\x12\x12\n\nrequest_id\x18\x02 \x01(\t\"\x8a\x01\n\x0bTrainingSet\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x42\n\x06status\x18\x02 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12\x17\n\x0f\x64\x65\x66\x61ult_variant\x18\x03 \x01(\t\x12\x10\n\x08variants\x18\x04 \x03(\t\"\xfa\x06\n\x12TrainingSetVariant\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07variant\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\r\n\x05owner\x18\x04 \x01(\t\x12+\n\x07\x63reated\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x10\n\x08provider\x18\x06 \x01(\t\x12\x42\n\x06status\x18\x07 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12\x41\n\x08\x66\x65\x61tures\x18\x08 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12>\n\x05label\x18\t \x01(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x30\n\x0clast_updated\x18\r \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x10\n\x08schedule\x18\x0e \x01(\t\x12\x44\n\x0c\x66\x65\x61ture_lags\x18\x0f \x03(\x0b\x32..featureform.serving.metadata.proto.FeatureLag\x12\x36\n\x04tags\x18\x10 \x01(\x0b\x32(.featureform.serving.metadata.proto.Tags\x12\x42\n\nproperties\x18\x11 \x01(\x0b\x32..featureform.serving.metadata.proto.Properties\x12\x13\n\x07task_id\x18\x12 \x01(\tB\x02\x18\x01\x12\x14\n\x0ctask_id_list\x18\x13 \x03(\t\x12^\n\x19resource_snowflake_config\x18\x14 \x01(\x0b\x32;.featureform.serving.metadata.proto.ResourceSnowflakeConfig\x12\x16\n\nis_deleted\x18\x15 \x01(\x08\x42\x02\x18\x01\x12/\n\x07\x64\x65leted\x18\x16 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x02\x18\x01\x12\x41\n\x04type\x18\x17 \x01(\x0e\x32\x33.featureform.serving.metadata.proto.TrainingSetType\"\x85\x01\n\x19TrainingSetVariantRequest\x12T\n\x14training_set_variant\x18\x01 \x01(\x0b\x32\x36.featureform.serving.metadata.proto.TrainingSetVariant\x12\x12\n\nrequest_id\x18\x02 \x01(\t\"\xcd\x03\n\x06\x45ntity\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x42\n\x06status\x18\x03 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12\x41\n\x08\x66\x65\x61tures\x18\x04 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12?\n\x06labels\x18\x05 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x45\n\x0ctrainingsets\x18\x06 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x36\n\x04tags\x18\x07 \x01(\x0b\x32(.featureform.serving.metadata.proto.Tags\x12\x42\n\nproperties\x18\x08 \x01(\x0b\x32..featureform.serving.metadata.proto.Properties\x12\x15\n\rfeature_views\x18\t \x03(\t\"_\n\rEntityRequest\x12:\n\x06\x65ntity\x18\x01 \x01(\x0b\x32*.featureform.serving.metadata.proto.Entity\x12\x12\n\nrequest_id\x18\x02 \x01(\t\"\xf1\x02\n\x05Model\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x41\n\x08\x66\x65\x61tures\x18\x03 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12?\n\x06labels\x18\x04 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x45\n\x0ctrainingsets\x18\x05 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x36\n\x04tags\x18\x06 \x01(\x0b\x32(.featureform.serving.metadata.proto.Tags\x12\x42\n\nproperties\x18\x07 \x01(\x0b\x32..featureform.serving.metadata.proto.Properties\"\\\n\x0cModelRequest\x12\x38\n\x05model\x18\x01 \x01(\x0b\x32).featureform.serving.metadata.proto.Model\x12\x12\n\nrequest_id\x18\x02 \x01(\t\"\x8c\x02\n\rStreamChannel\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x14\n\x0c\x63hannel_name\x18\x02 \x01(\t\x12\x42\n\x06status\x18\x03 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12\x10\n\x08provider\x18\x04 \x01(\t\x12\r\n\x05owner\x18\x05 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x06 \x01(\t\x12+\n\x07\x63reated\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x30\n\x0clast_updated\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"u\n\x14StreamChannelRequest\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12I\n\x0estream_channel\x18\x02 \x01(\x0b\x32\x31.featureform.serving.metadata.proto.StreamChannel\"\xf8\x03\n\x04User\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x42\n\x06status\x18\x02 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12\x41\n\x08\x66\x65\x61tures\x18\x03 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12?\n\x06labels\x18\x04 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x45\n\x0ctrainingsets\x18\x05 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12@\n\x07sources\x18\x06 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x36\n\x04tags\x18\x08 \x01(\x0b\x32(.featureform.serving.metadata.proto.Tags\x12\x42\n\nproperties\x18\t \x01(\x0b\x32..featureform.serving.metadata.proto.Properties\x12\x15\n\rfeature_views\x18\n \x03(\t\"Y\n\x0bUserRequest\x12\x36\n\x04user\x18\x01 \x01(\x0b\x32(.featureform.serving.metadata.proto.User\x12\x12\n\nrequest_id\x18\x02 \x01(\t\"\x85\x01\n\x06Source\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x42\n\x06status\x18\x02 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12\x17\n\x0f\x64\x65\x66\x61ult_variant\x18\x03 \x01(\t\x12\x10\n\x08variants\x18\x04 \x03(\t\"\xbc\x07\n\rSourceVariant\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07variant\x18\x02 \x01(\t\x12L\n\x0etransformation\x18\x0e \x01(\x0b\x32\x32.featureform.serving.metadata.proto.TransformationH\x00\x12\x46\n\x0bprimaryData\x18\x0f \x01(\x0b\x32/.featureform.serving.metadata.proto.PrimaryDataH\x00\x12\r\n\x05owner\x18\x04 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x05 \x01(\t\x12\x10\n\x08provider\x18\x06 \x01(\t\x12+\n\x07\x63reated\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x42\n\x06status\x18\x08 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12\r\n\x05table\x18\t \x01(\t\x12\x45\n\x0ctrainingsets\x18\n \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x41\n\x08\x66\x65\x61tures\x18\x0b \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12?\n\x06labels\x18\x0c \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x30\n\x0clast_updated\x18\r \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x10\n\x08schedule\x18\x10 \x01(\t\x12\x36\n\x04tags\x18\x11 \x01(\x0b\x32(.featureform.serving.metadata.proto.Tags\x12\x42\n\nproperties\x18\x12 \x01(\x0b\x32..featureform.serving.metadata.proto.Properties\x12\x13\n\x07task_id\x18\x13 \x01(\tB\x02\x18\x01\x12\x33\n\x10max_job_duration\x18\x14 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x14\n\x0ctask_id_list\x18\x15 \x03(\t\x12\x16\n\nis_deleted\x18\x16 \x01(\x08\x42\x02\x18\x01\x12/\n\x07\x64\x65leted\x18\x17 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x02\x18\x01\x42\x0c\n\ndefinition\"u\n\x14SourceVariantRequest\x12I\n\x0esource_variant\x18\x01 \x01(\x0b\x32\x31.featureform.serving.metadata.proto.SourceVariant\x12\x12\n\nrequest_id\x18\x02 \x01(\t\"(\n\nSparkParam\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\")\n\x0bWriteOption\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"+\n\rTableProperty\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\xe7\x01\n\nSparkFlags\x12\x44\n\x0cspark_params\x18\x01 \x03(\x0b\x32..featureform.serving.metadata.proto.SparkParam\x12\x46\n\rwrite_options\x18\x02 \x03(\x0b\x32/.featureform.serving.metadata.proto.WriteOption\x12K\n\x10table_properties\x18\x03 \x03(\x0b\x32\x31.featureform.serving.metadata.proto.TableProperty\"\xb6\x03\n\x08Location\x12=\n\x05table\x18\x01 \x01(\x0b\x32,.featureform.serving.metadata.proto.SQLTableH\x00\x12G\n\tfilestore\x18\x02 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.FileStoreTableH\x00\x12\x43\n\x07\x63\x61talog\x18\x03 \x01(\x0b\x32\x30.featureform.serving.metadata.proto.CatalogTableH\x00\x12>\n\x05kafka\x18\x04 \x01(\x0b\x32).featureform.serving.metadata.proto.KafkaB\x02\x18\x01H\x00\x12\x44\n\x06online\x18\x05 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.OnlineLocationH\x00\x12K\n\x0estream_channel\x18\x06 \x01(\x0b\x32\x31.featureform.serving.metadata.proto.StreamChannelH\x00\x42\n\n\x08location\"\xe7\x06\n\x0eTransformation\x12R\n\x11SQLTransformation\x18\x01 \x01(\x0b\x32\x35.featureform.serving.metadata.proto.SQLTransformationH\x00\x12P\n\x10\x44\x46Transformation\x18\x02 \x01(\x0b\x32\x34.featureform.serving.metadata.proto.DFTransformationH\x00\x12M\n\x0fkubernetes_args\x18\x03 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.KubernetesArgsH\x01\x12J\n\rHashPartition\x18\x04 \x01(\x0b\x32\x31.featureform.serving.metadata.proto.HashPartitionH\x02\x12L\n\x0e\x44\x61ilyPartition\x18\x05 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.DailyPartitionH\x02\x12\x43\n\x0bspark_flags\x18\x06 \x01(\x0b\x32..featureform.serving.metadata.proto.SparkFlags\x12=\n\x05table\x18\x07 \x01(\x0b\x32,.featureform.serving.metadata.proto.SQLTableH\x03\x12G\n\tfilestore\x18\x08 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.FileStoreTableH\x03\x12\x43\n\x07\x63\x61talog\x18\t \x01(\x0b\x32\x30.featureform.serving.metadata.proto.CatalogTableH\x03\x12>\n\x05kafka\x18\n \x01(\x0b\x32).featureform.serving.metadata.proto.KafkaB\x02\x18\x01H\x03\x12K\n\x0estream_channel\x18\x0b \x01(\x0b\x32\x31.featureform.serving.metadata.proto.StreamChannelH\x03\x42\x06\n\x04typeB\x06\n\x04\x61rgsB\x0b\n\tpartitionB\n\n\x08location\"0\n\rHashPartition\x12\x0e\n\x06\x63olumn\x18\x01 \x01(\t\x12\x0f\n\x07\x62uckets\x18\x02 \x01(\x05\" \n\x0e\x44\x61ilyPartition\x12\x0e\n\x06\x63olumn\x18\x01 \x01(\t\"o\n\x17KubernetesResourceSpecs\x12\x13\n\x0b\x63pu_request\x18\x01 \x01(\t\x12\x11\n\tcpu_limit\x18\x02 \x01(\t\x12\x16\n\x0ememory_request\x18\x03 \x01(\t\x12\x14\n\x0cmemory_limit\x18\x04 \x01(\t\"r\n\x0eKubernetesArgs\x12\x14\n\x0c\x64ocker_image\x18\x01 \x01(\t\x12J\n\x05specs\x18\x02 \x01(\x0b\x32;.featureform.serving.metadata.proto.KubernetesResourceSpecs\"\x8a\x03\n\x11SQLTransformation\x12\r\n\x05query\x18\x01 \x01(\t\x12?\n\x06source\x18\x02 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x16\n\x0eis_incremental\x18\x03 \x01(\x08\x12K\n\x12incremental_source\x18\x04 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12^\n\x19resource_snowflake_config\x18\x05 \x01(\x0b\x32;.featureform.serving.metadata.proto.ResourceSnowflakeConfig\x12\x14\n\x0cis_streaming\x18\x06 \x01(\x08\x12J\n\x11streaming_sources\x18\x07 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\"\xd9\x02\n\x10\x44\x46Transformation\x12\r\n\x05query\x18\x01 \x01(\x0c\x12?\n\x06inputs\x18\x02 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x13\n\x0bsource_text\x18\x03 \x01(\t\x12\x16\n\x0eis_incremental\x18\x04 \x01(\x08\x12L\n\x13incremental_sources\x18\x05 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x14\n\x0cis_streaming\x18\x06 \x01(\x08\x12G\n\x0estream_sources\x18\x07 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x1b\n\x13\x63\x61nonical_func_text\x18\x08 \x01(\t\"\x8d\x03\n\x0bPrimaryData\x12=\n\x05table\x18\x01 \x01(\x0b\x32,.featureform.serving.metadata.proto.SQLTableH\x00\x12G\n\tfilestore\x18\x03 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.FileStoreTableH\x00\x12\x43\n\x07\x63\x61talog\x18\x04 \x01(\x0b\x32\x30.featureform.serving.metadata.proto.CatalogTableH\x00\x12>\n\x05kafka\x18\x05 \x01(\x0b\x32).featureform.serving.metadata.proto.KafkaB\x02\x18\x01H\x00\x12K\n\x0estream_channel\x18\x06 \x01(\x0b\x32\x31.featureform.serving.metadata.proto.StreamChannelH\x00\x12\x18\n\x10timestamp_column\x18\x02 \x01(\tB\n\n\x08location\":\n\x08SQLTable\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08\x64\x61tabase\x18\x02 \x01(\t\x12\x0e\n\x06schema\x18\x03 \x01(\t\"\x1e\n\x0e\x46ileStoreTable\x12\x0c\n\x04path\x18\x01 \x01(\t\"\x16\n\x05Kafka\x12\r\n\x05topic\x18\x01 \x01(\t\"\x9c\x01\n\x0c\x43\x61talogTable\x12\x10\n\x08\x64\x61tabase\x18\x01 \x01(\t\x12\r\n\x05table\x18\x02 \x01(\t\x12\x14\n\x0ctable_format\x18\x03 \x01(\t\x12\x0e\n\x06schema\x18\x04 \x01(\t\x12\x45\n\x0c\x63\x61talog_type\x18\x05 \x01(\x0e\x32/.featureform.serving.metadata.proto.CatalogType\"\x13\n\x04Tags\x12\x0b\n\x03tag\x18\x01 \x03(\t\"+\n\x08Property\x12\x16\n\x0cstring_value\x18\x01 \x01(\tH\x00\x42\x07\n\x05value\"\xbb\x01\n\nProperties\x12N\n\x08property\x18\x01 \x03(\x0b\x32<.featureform.serving.metadata.proto.Properties.PropertyEntry\x1a]\n\rPropertyEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12;\n\x05value\x18\x02 \x01(\x0b\x32,.featureform.serving.metadata.proto.Property:\x02\x38\x01\"\x7f\n\x17StreamingFeatureVariant\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07variant\x18\x02 \x01(\t\x12\x0e\n\x06\x65ntity\x18\x03 \x01(\t\x12\r\n\x05value\x18\x04 \x01(\t\x12&\n\x02ts\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"}\n\x15StreamingLabelVariant\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07variant\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\t\x12\x0e\n\x06\x65ntity\x18\x04 \x01(\t\x12&\n\x02ts\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\x8b\x01\n\x17ResourceSnowflakeConfig\x12]\n\x14\x64ynamic_table_config\x18\x01 \x01(\x0b\x32?.featureform.serving.metadata.proto.SnowflakeDynamicTableConfig\x12\x11\n\twarehouse\x18\x02 \x01(\t\"\xbc\x01\n\x1bSnowflakeDynamicTableConfig\x12\x12\n\ntarget_lag\x18\x01 \x01(\t\x12\x45\n\x0crefresh_mode\x18\x02 \x01(\x0e\x32/.featureform.serving.metadata.proto.RefreshMode\x12\x42\n\ninitialize\x18\x03 \x01(\x0e\x32..featureform.serving.metadata.proto.Initialize\"]\n\x16MarkForDeletionRequest\x12\x43\n\x0bresource_id\x18\x01 \x01(\x0b\x32..featureform.serving.metadata.proto.ResourceID\"\x19\n\x17MarkForDeletionResponse\"^\n\x17\x46inalizeDeletionRequest\x12\x43\n\x0bresource_id\x18\x01 \x01(\x0b\x32..featureform.serving.metadata.proto.ResourceID\"\x1a\n\x18\x46inalizeDeletionResponse\"j\n#GetStagedForDeletionResourceRequest\x12\x43\n\x0bresource_id\x18\x01 \x01(\x0b\x32..featureform.serving.metadata.proto.ResourceID\"u\n$GetStagedForDeletionResourceResponse\x12M\n\x10resource_variant\x18\x01 \x01(\x0b\x32\x33.featureform.serving.metadata.proto.ResourceVariant\"[\n\x14PruneResourceRequest\x12\x43\n\x0bresource_id\x18\x01 \x01(\x0b\x32..featureform.serving.metadata.proto.ResourceID\"\xb6\x01\n\x15PruneResourceResponse\x12\x12\n\ntarget_lag\x18\x01 \x01(\t\x12\x45\n\x0crefresh_mode\x18\x02 \x01(\x0e\x32/.featureform.serving.metadata.proto.RefreshMode\x12\x42\n\ninitialize\x18\x03 \x01(\x0e\x32..featureform.serving.metadata.proto.Initialize\"\xc6\x01\n GetDownstreamDependenciesRequest\x12\x43\n\x0bresource_id\x18\x01 \x01(\x0b\x32..featureform.serving.metadata.proto.ResourceID\x12I\n\x0f\x64\x65pendency_type\x18\x02 \x01(\x0e\x32\x30.featureform.serving.metadata.proto.ResourceType\x12\x12\n\nrequest_id\x18\x03 \x01(\t\"\xb7\x05\n!GetDownstreamDependenciesResponse\x12M\n\x0f\x66\x65\x61ture_variant\x18\x01 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.FeatureVariantH\x00\x12I\n\rlabel_variant\x18\x02 \x01(\x0b\x32\x30.featureform.serving.metadata.proto.LabelVariantH\x00\x12K\n\x0esource_variant\x18\x03 \x01(\x0b\x32\x31.featureform.serving.metadata.proto.SourceVariantH\x00\x12V\n\x14training_set_variant\x18\x04 \x01(\x0b\x32\x36.featureform.serving.metadata.proto.TrainingSetVariantH\x00\x12:\n\x05model\x18\x05 \x01(\x0b\x32).featureform.serving.metadata.proto.ModelH\x00\x12\x38\n\x04user\x18\x06 \x01(\x0b\x32(.featureform.serving.metadata.proto.UserH\x00\x12@\n\x08provider\x18\x07 \x01(\x0b\x32,.featureform.serving.metadata.proto.ProviderH\x00\x12<\n\x06\x65ntity\x18\x08 \x01(\x0b\x32*.featureform.serving.metadata.proto.EntityH\x00\x12G\n\x0c\x66\x65\x61ture_view\x18\t \x01(\x0b\x32/.featureform.serving.metadata.proto.FeatureViewH\x00\x42\x14\n\x12\x64\x65pendent_resource\"~\n#GetAllDownstreamDependenciesRequest\x12\x43\n\x0bresource_id\x18\x01 \x01(\x0b\x32..featureform.serving.metadata.proto.ResourceID\x12\x12\n\nrequest_id\x18\x02 \x01(\t\"Y\n\x0c\x43olumnSchema\x12\x0c\n\x04name\x18\x01 \x01(\t\x12;\n\x04type\x18\x02 \x01(\x0b\x32-.featureform.serving.metadata.proto.ValueType\"\x9f\x01\n\rFeatureColumn\x12\x45\n\x0b\x66\x65\x61ture_col\x18\x01 \x01(\x0b\x32\x30.featureform.serving.metadata.proto.ColumnSchema\x12G\n\rtimestamp_col\x18\x02 \x01(\x0b\x32\x30.featureform.serving.metadata.proto.ColumnSchema\"\x9f\x01\n\x0e\x46\x65\x61turesSchema\x12\x44\n\nentity_col\x18\x01 \x01(\x0b\x32\x30.featureform.serving.metadata.proto.ColumnSchema\x12G\n\x0c\x66\x65\x61ture_cols\x18\x02 \x03(\x0b\x32\x31.featureform.serving.metadata.proto.FeatureColumn\"t\n\x0eOnlineLocation\x12\r\n\x05table\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\x05\x12\x42\n\x06schema\x18\x03 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.FeaturesSchema\"\xc0\x04\n\x0b\x46\x65\x61tureView\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0e\n\x06\x65ntity\x18\x02 \x01(\t\x12\x41\n\x08\x66\x65\x61tures\x18\x03 \x03(\x0b\x32/.featureform.serving.metadata.proto.NameVariant\x12\x10\n\x08provider\x18\x04 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x05 \x01(\t\x12R\n\rtable_options\x18\x06 \x01(\x0b\x32;.featureform.serving.metadata.proto.FeatureViewTableOptions\x12\r\n\x05owner\x18\x07 \x01(\t\x12\x42\n\x06status\x18\x08 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.ResourceStatus\x12\x14\n\x0ctask_id_list\x18\t \x03(\t\x12+\n\x07\x63reated\x18\n \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x30\n\x0clast_updated\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x46\n\x10offline_location\x18\x0c \x01(\x0b\x32,.featureform.serving.metadata.proto.Location\x12\x45\n\x0fonline_location\x18\r \x01(\x0b\x32,.featureform.serving.metadata.proto.Location\"\x19\n\x17\x46\x65\x61tureViewTableOptions\"#\n!FeatureViewMaterializationOptions\"\xa0\x02\n\x12\x46\x65\x61tureViewRequest\x12\x45\n\x0c\x66\x65\x61ture_view\x18\x01 \x01(\x0b\x32/.featureform.serving.metadata.proto.FeatureView\x12\x66\n\x17materialization_options\x18\x02 \x01(\x0b\x32\x45.featureform.serving.metadata.proto.FeatureViewMaterializationOptions\x12\x1e\n\x16materialization_engine\x18\x03 \x01(\t\x12\'\n\x1fuse_super_admin_delete_override\x18\x04 \x01(\x08\x12\x12\n\nrequest_id\x18\x05 \x01(\t\"\xd3\x01\n FeatureViewLocationUpdateRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x46\n\x10offline_location\x18\x02 \x01(\x0b\x32,.featureform.serving.metadata.proto.Location\x12\x45\n\x0fonline_location\x18\x03 \x01(\x0b\x32,.featureform.serving.metadata.proto.Location\x12\x12\n\nrequest_id\x18\x04 \x01(\t\"\xc4\x01\n\x1eGetUpstreamDependenciesRequest\x12\x43\n\x0bresource_id\x18\x01 \x01(\x0b\x32..featureform.serving.metadata.proto.ResourceID\x12I\n\x0f\x64\x65pendency_type\x18\x02 \x01(\x0e\x32\x30.featureform.serving.metadata.proto.ResourceType\x12\x12\n\nrequest_id\x18\x03 \x01(\t\"\xb5\x05\n\x1fGetUpstreamDependenciesResponse\x12M\n\x0f\x66\x65\x61ture_variant\x18\x01 \x01(\x0b\x32\x32.featureform.serving.metadata.proto.FeatureVariantH\x00\x12I\n\rlabel_variant\x18\x02 \x01(\x0b\x32\x30.featureform.serving.metadata.proto.LabelVariantH\x00\x12K\n\x0esource_variant\x18\x03 \x01(\x0b\x32\x31.featureform.serving.metadata.proto.SourceVariantH\x00\x12V\n\x14training_set_variant\x18\x04 \x01(\x0b\x32\x36.featureform.serving.metadata.proto.TrainingSetVariantH\x00\x12:\n\x05model\x18\x05 \x01(\x0b\x32).featureform.serving.metadata.proto.ModelH\x00\x12\x38\n\x04user\x18\x06 \x01(\x0b\x32(.featureform.serving.metadata.proto.UserH\x00\x12@\n\x08provider\x18\x07 \x01(\x0b\x32,.featureform.serving.metadata.proto.ProviderH\x00\x12<\n\x06\x65ntity\x18\x08 \x01(\x0b\x32*.featureform.serving.metadata.proto.EntityH\x00\x12G\n\x0c\x66\x65\x61ture_view\x18\t \x01(\x0b\x32/.featureform.serving.metadata.proto.FeatureViewH\x00\x42\x14\n\x12\x64\x65pendent_resource*\xef\x01\n\x0cResourceType\x12\x0b\n\x07\x46\x45\x41TURE\x10\x00\x12\t\n\x05LABEL\x10\x01\x12\x10\n\x0cTRAINING_SET\x10\x02\x12\n\n\x06SOURCE\x10\x03\x12\x13\n\x0f\x46\x45\x41TURE_VARIANT\x10\x04\x12\x11\n\rLABEL_VARIANT\x10\x05\x12\x18\n\x14TRAINING_SET_VARIANT\x10\x06\x12\x12\n\x0eSOURCE_VARIANT\x10\x07\x12\x0c\n\x08PROVIDER\x10\x08\x12\n\n\x06\x45NTITY\x10\t\x12\t\n\x05MODEL\x10\n\x12\x08\n\x04USER\x10\x0b\x12\x10\n\x0c\x46\x45\x41TURE_VIEW\x10\x0c\x12\x12\n\x0eSTREAM_CHANNEL\x10\r*F\n\x0f\x43omputationMode\x12\x0f\n\x0bPRECOMPUTED\x10\x00\x12\x13\n\x0f\x43LIENT_COMPUTED\x10\x01\x12\r\n\tSTREAMING\x10\x02*\x82\x01\n\nScalarType\x12\x08\n\x04NULL\x10\x00\x12\x07\n\x03INT\x10\x01\x12\x0b\n\x07\x46LOAT32\x10\x02\x12\x0b\n\x07\x46LOAT64\x10\x03\x12\n\n\x06STRING\x10\x04\x12\x08\n\x04\x42OOL\x10\x05\x12\t\n\x05INT32\x10\x06\x12\t\n\x05INT64\x10\x07\x12\x0c\n\x08\x44\x41TETIME\x10\x08\x12\r\n\tTIMESTAMP\x10\t*%\n\x0bTableFormat\x12\x0b\n\x07ICEBERG\x10\x00\x12\t\n\x05\x44\x45LTA\x10\x01*\"\n\x0b\x43\x61talogType\x12\x08\n\x04GLUE\x10\x00\x12\t\n\x05UNITY\x10\x01*w\n\x0bRefreshMode\x12\x1c\n\x18REFRESH_MODE_UNSPECIFIED\x10\x00\x12\x15\n\x11REFRESH_MODE_AUTO\x10\x01\x12\x15\n\x11REFRESH_MODE_FULL\x10\x02\x12\x1c\n\x18REFRESH_MODE_INCREMENTAL\x10\x03*^\n\nInitialize\x12\x1a\n\x16INITIALIZE_UNSPECIFIED\x10\x00\x12\x18\n\x14INITIALIZE_ON_CREATE\x10\x01\x12\x1a\n\x16INITIALIZE_ON_SCHEDULE\x10\x02*\x8d\x01\n\x0fTrainingSetType\x12!\n\x1dTRAINING_SET_TYPE_UNSPECIFIED\x10\x00\x12\x1d\n\x19TRAINING_SET_TYPE_DYNAMIC\x10\x01\x12\x1c\n\x18TRAINING_SET_TYPE_STATIC\x10\x02\x12\x1a\n\x16TRAINING_SET_TYPE_VIEW\x10\x03\x32\xd4-\n\x08Metadata\x12j\n\rGetAuthConfig\x12).featureform.serving.metadata.proto.Empty\x1a..featureform.serving.metadata.proto.AuthConfig\x12h\n\nCreateUser\x12/.featureform.serving.metadata.proto.UserRequest\x1a).featureform.serving.metadata.proto.Empty\x12p\n\x0e\x43reateProvider\x12\x33.featureform.serving.metadata.proto.ProviderRequest\x1a).featureform.serving.metadata.proto.Empty\x12z\n\x13\x43reateSourceVariant\x12\x38.featureform.serving.metadata.proto.SourceVariantRequest\x1a).featureform.serving.metadata.proto.Empty\x12l\n\x0c\x43reateEntity\x12\x31.featureform.serving.metadata.proto.EntityRequest\x1a).featureform.serving.metadata.proto.Empty\x12|\n\x14\x43reateFeatureVariant\x12\x39.featureform.serving.metadata.proto.FeatureVariantRequest\x1a).featureform.serving.metadata.proto.Empty\x12x\n\x12\x43reateLabelVariant\x12\x37.featureform.serving.metadata.proto.LabelVariantRequest\x1a).featureform.serving.metadata.proto.Empty\x12\x84\x01\n\x18\x43reateTrainingSetVariant\x12=.featureform.serving.metadata.proto.TrainingSetVariantRequest\x1a).featureform.serving.metadata.proto.Empty\x12j\n\x0b\x43reateModel\x12\x30.featureform.serving.metadata.proto.ModelRequest\x1a).featureform.serving.metadata.proto.Empty\x12z\n\x13\x43reateStreamChannel\x12\x38.featureform.serving.metadata.proto.StreamChannelRequest\x1a).featureform.serving.metadata.proto.Empty\x12}\n\x15RequestScheduleChange\x12\x39.featureform.serving.metadata.proto.ScheduleChangeRequest\x1a).featureform.serving.metadata.proto.Empty\x12{\n\x16MaterializeFeatureView\x12\x36.featureform.serving.metadata.proto.FeatureViewRequest\x1a).featureform.serving.metadata.proto.Empty\x12i\n\x08GetUsers\x12/.featureform.serving.metadata.proto.NameRequest\x1a(.featureform.serving.metadata.proto.User(\x01\x30\x01\x12o\n\x0bGetFeatures\x12/.featureform.serving.metadata.proto.NameRequest\x1a+.featureform.serving.metadata.proto.Feature(\x01\x30\x01\x12\x84\x01\n\x12GetFeatureVariants\x12\x36.featureform.serving.metadata.proto.NameVariantRequest\x1a\x32.featureform.serving.metadata.proto.FeatureVariant(\x01\x30\x01\x12k\n\tGetLabels\x12/.featureform.serving.metadata.proto.NameRequest\x1a).featureform.serving.metadata.proto.Label(\x01\x30\x01\x12\x80\x01\n\x10GetLabelVariants\x12\x36.featureform.serving.metadata.proto.NameVariantRequest\x1a\x30.featureform.serving.metadata.proto.LabelVariant(\x01\x30\x01\x12w\n\x0fGetTrainingSets\x12/.featureform.serving.metadata.proto.NameRequest\x1a/.featureform.serving.metadata.proto.TrainingSet(\x01\x30\x01\x12\x8c\x01\n\x16GetTrainingSetVariants\x12\x36.featureform.serving.metadata.proto.NameVariantRequest\x1a\x36.featureform.serving.metadata.proto.TrainingSetVariant(\x01\x30\x01\x12m\n\nGetSources\x12/.featureform.serving.metadata.proto.NameRequest\x1a*.featureform.serving.metadata.proto.Source(\x01\x30\x01\x12\x82\x01\n\x11GetSourceVariants\x12\x36.featureform.serving.metadata.proto.NameVariantRequest\x1a\x31.featureform.serving.metadata.proto.SourceVariant(\x01\x30\x01\x12q\n\x0cGetProviders\x12/.featureform.serving.metadata.proto.NameRequest\x1a,.featureform.serving.metadata.proto.Provider(\x01\x30\x01\x12n\n\x0bGetEntities\x12/.featureform.serving.metadata.proto.NameRequest\x1a*.featureform.serving.metadata.proto.Entity(\x01\x30\x01\x12k\n\tGetModels\x12/.featureform.serving.metadata.proto.NameRequest\x1a).featureform.serving.metadata.proto.Model(\x01\x30\x01\x12{\n\x11GetStreamChannels\x12/.featureform.serving.metadata.proto.NameRequest\x1a\x31.featureform.serving.metadata.proto.StreamChannel(\x01\x30\x01\x12\xac\x01\n\x19GetDownstreamDependencies\x12\x44.featureform.serving.metadata.proto.GetDownstreamDependenciesRequest\x1a\x45.featureform.serving.metadata.proto.GetDownstreamDependenciesResponse(\x01\x30\x01\x12\x9b\x01\n\x1cGetAllDownstreamDependencies\x12G.featureform.serving.metadata.proto.GetAllDownstreamDependenciesRequest\x1a..featureform.serving.metadata.proto.ResourceID(\x01\x30\x01\x12w\n\x0fGetFeatureViews\x12/.featureform.serving.metadata.proto.NameRequest\x1a/.featureform.serving.metadata.proto.FeatureView(\x01\x30\x01\x12\xa6\x01\n\x17GetUpstreamDependencies\x12\x42.featureform.serving.metadata.proto.GetUpstreamDependenciesRequest\x1a\x43.featureform.serving.metadata.proto.GetUpstreamDependenciesResponse(\x01\x30\x01\x12\x8a\x01\n\x0fMarkForDeletion\x12:.featureform.serving.metadata.proto.MarkForDeletionRequest\x1a;.featureform.serving.metadata.proto.MarkForDeletionResponse\x12\x8d\x01\n\x10\x46inalizeDeletion\x12;.featureform.serving.metadata.proto.FinalizeDeletionRequest\x1a<.featureform.serving.metadata.proto.FinalizeDeletionResponse\x12\xb1\x01\n\x1cGetStagedForDeletionResource\x12G.featureform.serving.metadata.proto.GetStagedForDeletionResourceRequest\x1aH.featureform.serving.metadata.proto.GetStagedForDeletionResourceResponse\x12\x84\x01\n\rPruneResource\x12\x38.featureform.serving.metadata.proto.PruneResourceRequest\x1a\x39.featureform.serving.metadata.proto.PruneResourceResponse\x12~\n\rGetEquivalent\x12\x38.featureform.serving.metadata.proto.GetEquivalentRequest\x1a\x33.featureform.serving.metadata.proto.ResourceVariant\x12`\n\x03Run\x12..featureform.serving.metadata.proto.RunRequest\x1a).featureform.serving.metadata.proto.Empty\x12n\n\x0cListFeatures\x12/.featureform.serving.metadata.proto.ListRequest\x1a+.featureform.serving.metadata.proto.Feature0\x01\x12j\n\nListLabels\x12/.featureform.serving.metadata.proto.ListRequest\x1a).featureform.serving.metadata.proto.Label0\x01\x12v\n\x10ListTrainingSets\x12/.featureform.serving.metadata.proto.ListRequest\x1a/.featureform.serving.metadata.proto.TrainingSet0\x01\x12l\n\x0bListSources\x12/.featureform.serving.metadata.proto.ListRequest\x1a*.featureform.serving.metadata.proto.Source0\x01\x12h\n\tListUsers\x12/.featureform.serving.metadata.proto.ListRequest\x1a(.featureform.serving.metadata.proto.User0\x01\x12p\n\rListProviders\x12/.featureform.serving.metadata.proto.ListRequest\x1a,.featureform.serving.metadata.proto.Provider0\x01\x12m\n\x0cListEntities\x12/.featureform.serving.metadata.proto.ListRequest\x1a*.featureform.serving.metadata.proto.Entity0\x01\x12j\n\nListModels\x12/.featureform.serving.metadata.proto.ListRequest\x1a).featureform.serving.metadata.proto.Model0\x01\x12v\n\x10ListFeatureViews\x12/.featureform.serving.metadata.proto.ListRequest\x1a/.featureform.serving.metadata.proto.FeatureView0\x01\x12z\n\x12ListStreamChannels\x12/.featureform.serving.metadata.proto.ListRequest\x1a\x31.featureform.serving.metadata.proto.StreamChannel0\x01\x12t\n\x11SetResourceStatus\x12\x34.featureform.serving.metadata.proto.SetStatusRequest\x1a).featureform.serving.metadata.proto.Empty\x12\x8a\x01\n\x17SetFeatureViewLocations\x12\x44.featureform.serving.metadata.proto.FeatureViewLocationUpdateRequest\x1a).featureform.serving.metadata.proto.Empty2\xbe-\n\x03\x41pi\x12j\n\rGetAuthConfig\x12).featureform.serving.metadata.proto.Empty\x1a..featureform.serving.metadata.proto.AuthConfig\x12h\n\nCreateUser\x12/.featureform.serving.metadata.proto.UserRequest\x1a).featureform.serving.metadata.proto.Empty\x12p\n\x0e\x43reateProvider\x12\x33.featureform.serving.metadata.proto.ProviderRequest\x1a).featureform.serving.metadata.proto.Empty\x12z\n\x13\x43reateSourceVariant\x12\x38.featureform.serving.metadata.proto.SourceVariantRequest\x1a).featureform.serving.metadata.proto.Empty\x12l\n\x0c\x43reateEntity\x12\x31.featureform.serving.metadata.proto.EntityRequest\x1a).featureform.serving.metadata.proto.Empty\x12|\n\x14\x43reateFeatureVariant\x12\x39.featureform.serving.metadata.proto.FeatureVariantRequest\x1a).featureform.serving.metadata.proto.Empty\x12x\n\x12\x43reateLabelVariant\x12\x37.featureform.serving.metadata.proto.LabelVariantRequest\x1a).featureform.serving.metadata.proto.Empty\x12\x84\x01\n\x18\x43reateTrainingSetVariant\x12=.featureform.serving.metadata.proto.TrainingSetVariantRequest\x1a).featureform.serving.metadata.proto.Empty\x12j\n\x0b\x43reateModel\x12\x30.featureform.serving.metadata.proto.ModelRequest\x1a).featureform.serving.metadata.proto.Empty\x12z\n\x13\x43reateStreamChannel\x12\x38.featureform.serving.metadata.proto.StreamChannelRequest\x1a).featureform.serving.metadata.proto.Empty\x12}\n\x15RequestScheduleChange\x12\x39.featureform.serving.metadata.proto.ScheduleChangeRequest\x1a).featureform.serving.metadata.proto.Empty\x12{\n\x16MaterializeFeatureView\x12\x36.featureform.serving.metadata.proto.FeatureViewRequest\x1a).featureform.serving.metadata.proto.Empty\x12i\n\x08GetUsers\x12/.featureform.serving.metadata.proto.NameRequest\x1a(.featureform.serving.metadata.proto.User(\x01\x30\x01\x12o\n\x0bGetFeatures\x12/.featureform.serving.metadata.proto.NameRequest\x1a+.featureform.serving.metadata.proto.Feature(\x01\x30\x01\x12\x84\x01\n\x12GetFeatureVariants\x12\x36.featureform.serving.metadata.proto.NameVariantRequest\x1a\x32.featureform.serving.metadata.proto.FeatureVariant(\x01\x30\x01\x12k\n\tGetLabels\x12/.featureform.serving.metadata.proto.NameRequest\x1a).featureform.serving.metadata.proto.Label(\x01\x30\x01\x12\x80\x01\n\x10GetLabelVariants\x12\x36.featureform.serving.metadata.proto.NameVariantRequest\x1a\x30.featureform.serving.metadata.proto.LabelVariant(\x01\x30\x01\x12w\n\x0fGetTrainingSets\x12/.featureform.serving.metadata.proto.NameRequest\x1a/.featureform.serving.metadata.proto.TrainingSet(\x01\x30\x01\x12\x8c\x01\n\x16GetTrainingSetVariants\x12\x36.featureform.serving.metadata.proto.NameVariantRequest\x1a\x36.featureform.serving.metadata.proto.TrainingSetVariant(\x01\x30\x01\x12m\n\nGetSources\x12/.featureform.serving.metadata.proto.NameRequest\x1a*.featureform.serving.metadata.proto.Source(\x01\x30\x01\x12\x82\x01\n\x11GetSourceVariants\x12\x36.featureform.serving.metadata.proto.NameVariantRequest\x1a\x31.featureform.serving.metadata.proto.SourceVariant(\x01\x30\x01\x12q\n\x0cGetProviders\x12/.featureform.serving.metadata.proto.NameRequest\x1a,.featureform.serving.metadata.proto.Provider(\x01\x30\x01\x12n\n\x0bGetEntities\x12/.featureform.serving.metadata.proto.NameRequest\x1a*.featureform.serving.metadata.proto.Entity(\x01\x30\x01\x12k\n\tGetModels\x12/.featureform.serving.metadata.proto.NameRequest\x1a).featureform.serving.metadata.proto.Model(\x01\x30\x01\x12{\n\x11GetStreamChannels\x12/.featureform.serving.metadata.proto.NameRequest\x1a\x31.featureform.serving.metadata.proto.StreamChannel(\x01\x30\x01\x12\xac\x01\n\x19GetDownstreamDependencies\x12\x44.featureform.serving.metadata.proto.GetDownstreamDependenciesRequest\x1a\x45.featureform.serving.metadata.proto.GetDownstreamDependenciesResponse(\x01\x30\x01\x12\x9b\x01\n\x1cGetAllDownstreamDependencies\x12G.featureform.serving.metadata.proto.GetAllDownstreamDependenciesRequest\x1a..featureform.serving.metadata.proto.ResourceID(\x01\x30\x01\x12w\n\x0fGetFeatureViews\x12/.featureform.serving.metadata.proto.NameRequest\x1a/.featureform.serving.metadata.proto.FeatureView(\x01\x30\x01\x12\xa6\x01\n\x17GetUpstreamDependencies\x12\x42.featureform.serving.metadata.proto.GetUpstreamDependenciesRequest\x1a\x43.featureform.serving.metadata.proto.GetUpstreamDependenciesResponse(\x01\x30\x01\x12\x8a\x01\n\x0fMarkForDeletion\x12:.featureform.serving.metadata.proto.MarkForDeletionRequest\x1a;.featureform.serving.metadata.proto.MarkForDeletionResponse\x12\x8d\x01\n\x10\x46inalizeDeletion\x12;.featureform.serving.metadata.proto.FinalizeDeletionRequest\x1a<.featureform.serving.metadata.proto.FinalizeDeletionResponse\x12\xb1\x01\n\x1cGetStagedForDeletionResource\x12G.featureform.serving.metadata.proto.GetStagedForDeletionResourceRequest\x1aH.featureform.serving.metadata.proto.GetStagedForDeletionResourceResponse\x12\x84\x01\n\rPruneResource\x12\x38.featureform.serving.metadata.proto.PruneResourceRequest\x1a\x39.featureform.serving.metadata.proto.PruneResourceResponse\x12~\n\rGetEquivalent\x12\x38.featureform.serving.metadata.proto.GetEquivalentRequest\x1a\x33.featureform.serving.metadata.proto.ResourceVariant\x12`\n\x03Run\x12..featureform.serving.metadata.proto.RunRequest\x1a).featureform.serving.metadata.proto.Empty\x12n\n\x0cListFeatures\x12/.featureform.serving.metadata.proto.ListRequest\x1a+.featureform.serving.metadata.proto.Feature0\x01\x12j\n\nListLabels\x12/.featureform.serving.metadata.proto.ListRequest\x1a).featureform.serving.metadata.proto.Label0\x01\x12v\n\x10ListTrainingSets\x12/.featureform.serving.metadata.proto.ListRequest\x1a/.featureform.serving.metadata.proto.TrainingSet0\x01\x12l\n\x0bListSources\x12/.featureform.serving.metadata.proto.ListRequest\x1a*.featureform.serving.metadata.proto.Source0\x01\x12h\n\tListUsers\x12/.featureform.serving.metadata.proto.ListRequest\x1a(.featureform.serving.metadata.proto.User0\x01\x12p\n\rListProviders\x12/.featureform.serving.metadata.proto.ListRequest\x1a,.featureform.serving.metadata.proto.Provider0\x01\x12m\n\x0cListEntities\x12/.featureform.serving.metadata.proto.ListRequest\x1a*.featureform.serving.metadata.proto.Entity0\x01\x12j\n\nListModels\x12/.featureform.serving.metadata.proto.ListRequest\x1a).featureform.serving.metadata.proto.Model0\x01\x12y\n\rWriteFeatures\x12;.featureform.serving.metadata.proto.StreamingFeatureVariant\x1a).featureform.serving.metadata.proto.Empty(\x01\x12u\n\x0bWriteLabels\x12\x39.featureform.serving.metadata.proto.StreamingLabelVariant\x1a).featureform.serving.metadata.proto.Empty(\x01\x12v\n\x10ListFeatureViews\x12/.featureform.serving.metadata.proto.ListRequest\x1a/.featureform.serving.metadata.proto.FeatureView0\x01\x12z\n\x12ListStreamChannels\x12/.featureform.serving.metadata.proto.ListRequest\x1a\x31.featureform.serving.metadata.proto.StreamChannel0\x01\x42\'Z%github.com/featureform/metadata/protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'featureform.proto.metadata_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z%github.com/featureform/metadata/proto'
  _globals['_FEATUREVARIANT'].fields_by_name['task_id']._options = None
  _globals['_FEATUREVARIANT'].fields_by_name['task_id']._serialized_options = b'\030\001'
  _globals['_FEATUREVARIANT'].fields_by_name['is_deleted']._options = None
  _globals['_FEATUREVARIANT'].fields_by_name['is_deleted']._serialized_options = b'\030\001'
  _globals['_FEATUREVARIANT'].fields_by_name['deleted']._options = None
  _globals['_FEATUREVARIANT'].fields_by_name['deleted']._serialized_options = b'\030\001'
  _globals['_LABELVARIANT'].fields_by_name['task_id']._options = None
  _globals['_LABELVARIANT'].fields_by_name['task_id']._serialized_options = b'\030\001'
  _globals['_LABELVARIANT'].fields_by_name['is_deleted']._options = None
  _globals['_LABELVARIANT'].fields_by_name['is_deleted']._serialized_options = b'\030\001'
  _globals['_LABELVARIANT'].fields_by_name['deleted']._options = None
  _globals['_LABELVARIANT'].fields_by_name['deleted']._serialized_options = b'\030\001'
  _globals['_PROVIDER'].fields_by_name['is_deleted']._options = None
  _globals['_PROVIDER'].fields_by_name['is_deleted']._serialized_options = b'\030\001'
  _globals['_PROVIDER'].fields_by_name['deleted']._options = None
  _globals['_PROVIDER'].fields_by_name['deleted']._serialized_options = b'\030\001'
  _globals['_TRAININGSETVARIANT'].fields_by_name['task_id']._options = None
  _globals['_TRAININGSETVARIANT'].fields_by_name['task_id']._serialized_options = b'\030\001'
  _globals['_TRAININGSETVARIANT'].fields_by_name['is_deleted']._options = None
  _globals['_TRAININGSETVARIANT'].fields_by_name['is_deleted']._serialized_options = b'\030\001'
  _globals['_TRAININGSETVARIANT'].fields_by_name['deleted']._options = None
  _globals['_TRAININGSETVARIANT'].fields_by_name['deleted']._serialized_options = b'\030\001'
  _globals['_SOURCEVARIANT'].fields_by_name['task_id']._options = None
  _globals['_SOURCEVARIANT'].fields_by_name['task_id']._serialized_options = b'\030\001'
  _globals['_SOURCEVARIANT'].fields_by_name['is_deleted']._options = None
  _globals['_SOURCEVARIANT'].fields_by_name['is_deleted']._serialized_options = b'\030\001'
  _globals['_SOURCEVARIANT'].fields_by_name['deleted']._options = None
  _globals['_SOURCEVARIANT'].fields_by_name['deleted']._serialized_options = b'\030\001'
  _globals['_LOCATION'].fields_by_name['kafka']._options = None
  _globals['_LOCATION'].fields_by_name['kafka']._serialized_options = b'\030\001'
  _globals['_TRANSFORMATION'].fields_by_name['kafka']._options = None
  _globals['_TRANSFORMATION'].fields_by_name['kafka']._serialized_options = b'\030\001'
  _globals['_PRIMARYDATA'].fields_by_name['kafka']._options = None
  _globals['_PRIMARYDATA'].fields_by_name['kafka']._serialized_options = b'\030\001'
  _globals['_PROPERTIES_PROPERTYENTRY']._options = None
  _globals['_PROPERTIES_PROPERTYENTRY']._serialized_options = b'8\001'
  _globals['_RESOURCETYPE']._serialized_start=19889
  _globals['_RESOURCETYPE']._serialized_end=20128
  _globals['_COMPUTATIONMODE']._serialized_start=20130
  _globals['_COMPUTATIONMODE']._serialized_end=20200
  _globals['_SCALARTYPE']._serialized_start=20203
  _globals['_SCALARTYPE']._serialized_end=20333
  _globals['_TABLEFORMAT']._serialized_start=20335
  _globals['_TABLEFORMAT']._serialized_end=20372
  _globals['_CATALOGTYPE']._serialized_start=20374
  _globals['_CATALOGTYPE']._serialized_end=20408
  _globals['_REFRESHMODE']._serialized_start=20410
  _globals['_REFRESHMODE']._serialized_end=20529
  _globals['_INITIALIZE']._serialized_start=20531
  _globals['_INITIALIZE']._serialized_end=20625
  _globals['_TRAININGSETTYPE']._serialized_start=20628
  _globals['_TRAININGSETTYPE']._serialized_end=20769
  _globals['_OKTAAUTHCONFIG']._serialized_start=164
  _globals['_OKTAAUTHCONFIG']._serialized_end=271
  _globals['_PASSTHROUGHAUTHCONFIG']._serialized_start=273
  _globals['_PASSTHROUGHAUTHCONFIG']._serialized_end=296
  _globals['_CIDRAUTHCONFIG']._serialized_start=298
  _globals['_CIDRAUTHCONFIG']._serialized_end=314
  _globals['_AUTHCONFIG']._serialized_start=317
  _globals['_AUTHCONFIG']._serialized_end=558
  _globals['_NAME']._serialized_start=560
  _globals['_NAME']._serialized_end=580
  _globals['_NAMEREQUEST']._serialized_start=582
  _globals['_NAMEREQUEST']._serialized_end=671
  _globals['_ERRORSTATUS']._serialized_start=673
  _globals['_ERRORSTATUS']._serialized_end=756
  _globals['_RESOURCESTATUS']._serialized_start=759
  _globals['_RESOURCESTATUS']._serialized_end=1046
  _globals['_RESOURCESTATUS_STATUS']._serialized_start=946
  _globals['_RESOURCESTATUS_STATUS']._serialized_end=1046
  _globals['_RESOURCEID']._serialized_start=1049
  _globals['_RESOURCEID']._serialized_end=1201
  _globals['_SETSTATUSREQUEST']._serialized_start=1204
  _globals['_SETSTATUSREQUEST']._serialized_end=1359
  _globals['_SCHEDULECHANGEREQUEST']._serialized_start=1361
  _globals['_SCHEDULECHANGEREQUEST']._serialized_end=1471
  _globals['_NAMEVARIANT']._serialized_start=1473
  _globals['_NAMEVARIANT']._serialized_end=1517
  _globals['_NAMEVARIANTREQUEST']._serialized_start=1519
  _globals['_NAMEVARIANTREQUEST']._serialized_end=1630
  _globals['_EMPTY']._serialized_start=1632
  _globals['_EMPTY']._serialized_end=1639
  _globals['_LISTREQUEST']._serialized_start=1641
  _globals['_LISTREQUEST']._serialized_end=1674
  _globals['_FEATURE']._serialized_start=1677
  _globals['_FEATURE']._serialized_end=1811
  _globals['_COLUMNS']._serialized_start=1813
  _globals['_COLUMNS']._serialized_end=1865
  _globals['_PYTHONFUNCTION']._serialized_start=1867
  _globals['_PYTHONFUNCTION']._serialized_end=1898
  _globals['_STREAM']._serialized_start=1900
  _globals['_STREAM']._serialized_end=1934
  _globals['_RESOURCEVARIANT']._serialized_start=1937
  _globals['_RESOURCEVARIANT']._serialized_end=2358
  _globals['_GETEQUIVALENTREQUEST']._serialized_start=2360
  _globals['_GETEQUIVALENTREQUEST']._serialized_end=2472
  _globals['_RUNREQUEST']._serialized_start=2474
  _globals['_RUNREQUEST']._serialized_end=2577
  _globals['_FEATUREVARIANT']._serialized_start=2580
  _globals['_FEATUREVARIANT']._serialized_end=3908
  _globals['_FEATUREVARIANTREQUEST']._serialized_start=3910
  _globals['_FEATUREVARIANTREQUEST']._serialized_end=4030
  _globals['_VALUETYPE']._serialized_start=4033
  _globals['_VALUETYPE']._serialized_end=4184
  _globals['_VECTORTYPE']._serialized_start=4186
  _globals['_VECTORTYPE']._serialized_end=4303
  _globals['_FEATUREPARAMETERS']._serialized_start=4306
  _globals['_FEATUREPARAMETERS']._serialized_end=4513
  _globals['_PRECOMPUTEDFEATUREPARAMETERS']._serialized_start=4515
  _globals['_PRECOMPUTEDFEATUREPARAMETERS']._serialized_end=4545
  _globals['_ONDEMANDFEATUREPARAMETERS']._serialized_start=4547
  _globals['_ONDEMANDFEATUREPARAMETERS']._serialized_end=4594
  _globals['_FEATURELAG']._serialized_start=4596
  _globals['_FEATURELAG']._serialized_end=4696
  _globals['_LABEL']._serialized_start=4699
  _globals['_LABEL']._serialized_end=4831
  _globals['_LABELVARIANT']._serialized_start=4834
  _globals['_LABELVARIANT']._serialized_end=5812
  _globals['_ENTITYMAPPINGS']._serialized_start=5815
  _globals['_ENTITYMAPPINGS']._serialized_end=5948
  _globals['_ENTITYMAPPING']._serialized_start=5950
  _globals['_ENTITYMAPPING']._serialized_end=6002
  _globals['_LABELVARIANTREQUEST']._serialized_start=6004
  _globals['_LABELVARIANTREQUEST']._serialized_end=6118
  _globals['_PROVIDER']._serialized_start=6121
  _globals['_PROVIDER']._serialized_end=6773
  _globals['_PROVIDERREQUEST']._serialized_start=6775
  _globals['_PROVIDERREQUEST']._serialized_end=6876
  _globals['_TRAININGSET']._serialized_start=6879
  _globals['_TRAININGSET']._serialized_end=7017
  _globals['_TRAININGSETVARIANT']._serialized_start=7020
  _globals['_TRAININGSETVARIANT']._serialized_end=7910
  _globals['_TRAININGSETVARIANTREQUEST']._serialized_start=7913
  _globals['_TRAININGSETVARIANTREQUEST']._serialized_end=8046
  _globals['_ENTITY']._serialized_start=8049
  _globals['_ENTITY']._serialized_end=8510
  _globals['_ENTITYREQUEST']._serialized_start=8512
  _globals['_ENTITYREQUEST']._serialized_end=8607
  _globals['_MODEL']._serialized_start=8610
  _globals['_MODEL']._serialized_end=8979
  _globals['_MODELREQUEST']._serialized_start=8981
  _globals['_MODELREQUEST']._serialized_end=9073
  _globals['_STREAMCHANNEL']._serialized_start=9076
  _globals['_STREAMCHANNEL']._serialized_end=9344
  _globals['_STREAMCHANNELREQUEST']._serialized_start=9346
  _globals['_STREAMCHANNELREQUEST']._serialized_end=9463
  _globals['_USER']._serialized_start=9466
  _globals['_USER']._serialized_end=9970
  _globals['_USERREQUEST']._serialized_start=9972
  _globals['_USERREQUEST']._serialized_end=10061
  _globals['_SOURCE']._serialized_start=10064
  _globals['_SOURCE']._serialized_end=10197
  _globals['_SOURCEVARIANT']._serialized_start=10200
  _globals['_SOURCEVARIANT']._serialized_end=11156
  _globals['_SOURCEVARIANTREQUEST']._serialized_start=11158
  _globals['_SOURCEVARIANTREQUEST']._serialized_end=11275
  _globals['_SPARKPARAM']._serialized_start=11277
  _globals['_SPARKPARAM']._serialized_end=11317
  _globals['_WRITEOPTION']._serialized_start=11319
  _globals['_WRITEOPTION']._serialized_end=11360
  _globals['_TABLEPROPERTY']._serialized_start=11362
  _globals['_TABLEPROPERTY']._serialized_end=11405
  _globals['_SPARKFLAGS']._serialized_start=11408
  _globals['_SPARKFLAGS']._serialized_end=11639
  _globals['_LOCATION']._serialized_start=11642
  _globals['_LOCATION']._serialized_end=12080
  _globals['_TRANSFORMATION']._serialized_start=12083
  _globals['_TRANSFORMATION']._serialized_end=12954
  _globals['_HASHPARTITION']._serialized_start=12956
  _globals['_HASHPARTITION']._serialized_end=13004
  _globals['_DAILYPARTITION']._serialized_start=13006
  _globals['_DAILYPARTITION']._serialized_end=13038
  _globals['_KUBERNETESRESOURCESPECS']._serialized_start=13040
  _globals['_KUBERNETESRESOURCESPECS']._serialized_end=13151
  _globals['_KUBERNETESARGS']._serialized_start=13153
  _globals['_KUBERNETESARGS']._serialized_end=13267
  _globals['_SQLTRANSFORMATION']._serialized_start=13270
  _globals['_SQLTRANSFORMATION']._serialized_end=13664
  _globals['_DFTRANSFORMATION']._serialized_start=13667
  _globals['_DFTRANSFORMATION']._serialized_end=14012
  _globals['_PRIMARYDATA']._serialized_start=14015
  _globals['_PRIMARYDATA']._serialized_end=14412
  _globals['_SQLTABLE']._serialized_start=14414
  _globals['_SQLTABLE']._serialized_end=14472
  _globals['_FILESTORETABLE']._serialized_start=14474
  _globals['_FILESTORETABLE']._serialized_end=14504
  _globals['_KAFKA']._serialized_start=14506
  _globals['_KAFKA']._serialized_end=14528
  _globals['_CATALOGTABLE']._serialized_start=14531
  _globals['_CATALOGTABLE']._serialized_end=14687
  _globals['_TAGS']._serialized_start=14689
  _globals['_TAGS']._serialized_end=14708
  _globals['_PROPERTY']._serialized_start=14710
  _globals['_PROPERTY']._serialized_end=14753
  _globals['_PROPERTIES']._serialized_start=14756
  _globals['_PROPERTIES']._serialized_end=14943
  _globals['_PROPERTIES_PROPERTYENTRY']._serialized_start=14850
  _globals['_PROPERTIES_PROPERTYENTRY']._serialized_end=14943
  _globals['_STREAMINGFEATUREVARIANT']._serialized_start=14945
  _globals['_STREAMINGFEATUREVARIANT']._serialized_end=15072
  _globals['_STREAMINGLABELVARIANT']._serialized_start=15074
  _globals['_STREAMINGLABELVARIANT']._serialized_end=15199
  _globals['_RESOURCESNOWFLAKECONFIG']._serialized_start=15202
  _globals['_RESOURCESNOWFLAKECONFIG']._serialized_end=15341
  _globals['_SNOWFLAKEDYNAMICTABLECONFIG']._serialized_start=15344
  _globals['_SNOWFLAKEDYNAMICTABLECONFIG']._serialized_end=15532
  _globals['_MARKFORDELETIONREQUEST']._serialized_start=15534
  _globals['_MARKFORDELETIONREQUEST']._serialized_end=15627
  _globals['_MARKFORDELETIONRESPONSE']._serialized_start=15629
  _globals['_MARKFORDELETIONRESPONSE']._serialized_end=15654
  _globals['_FINALIZEDELETIONREQUEST']._serialized_start=15656
  _globals['_FINALIZEDELETIONREQUEST']._serialized_end=15750
  _globals['_FINALIZEDELETIONRESPONSE']._serialized_start=15752
  _globals['_FINALIZEDELETIONRESPONSE']._serialized_end=15778
  _globals['_GETSTAGEDFORDELETIONRESOURCEREQUEST']._serialized_start=15780
  _globals['_GETSTAGEDFORDELETIONRESOURCEREQUEST']._serialized_end=15886
  _globals['_GETSTAGEDFORDELETIONRESOURCERESPONSE']._serialized_start=15888
  _globals['_GETSTAGEDFORDELETIONRESOURCERESPONSE']._serialized_end=16005
  _globals['_PRUNERESOURCEREQUEST']._serialized_start=16007
  _globals['_PRUNERESOURCEREQUEST']._serialized_end=16098
  _globals['_PRUNERESOURCERESPONSE']._serialized_start=16101
  _globals['_PRUNERESOURCERESPONSE']._serialized_end=16283
  _globals['_GETDOWNSTREAMDEPENDENCIESREQUEST']._serialized_start=16286
  _globals['_GETDOWNSTREAMDEPENDENCIESREQUEST']._serialized_end=16484
  _globals['_GETDOWNSTREAMDEPENDENCIESRESPONSE']._serialized_start=16487
  _globals['_GETDOWNSTREAMDEPENDENCIESRESPONSE']._serialized_end=17182
  _globals['_GETALLDOWNSTREAMDEPENDENCIESREQUEST']._serialized_start=17184
  _globals['_GETALLDOWNSTREAMDEPENDENCIESREQUEST']._serialized_end=17310
  _globals['_COLUMNSCHEMA']._serialized_start=17312
  _globals['_COLUMNSCHEMA']._serialized_end=17401
  _globals['_FEATURECOLUMN']._serialized_start=17404
  _globals['_FEATURECOLUMN']._serialized_end=17563
  _globals['_FEATURESSCHEMA']._serialized_start=17566
  _globals['_FEATURESSCHEMA']._serialized_end=17725
  _globals['_ONLINELOCATION']._serialized_start=17727
  _globals['_ONLINELOCATION']._serialized_end=17843
  _globals['_FEATUREVIEW']._serialized_start=17846
  _globals['_FEATUREVIEW']._serialized_end=18422
  _globals['_FEATUREVIEWTABLEOPTIONS']._serialized_start=18424
  _globals['_FEATUREVIEWTABLEOPTIONS']._serialized_end=18449
  _globals['_FEATUREVIEWMATERIALIZATIONOPTIONS']._serialized_start=18451
  _globals['_FEATUREVIEWMATERIALIZATIONOPTIONS']._serialized_end=18486
  _globals['_FEATUREVIEWREQUEST']._serialized_start=18489
  _globals['_FEATUREVIEWREQUEST']._serialized_end=18777
  _globals['_FEATUREVIEWLOCATIONUPDATEREQUEST']._serialized_start=18780
  _globals['_FEATUREVIEWLOCATIONUPDATEREQUEST']._serialized_end=18991
  _globals['_GETUPSTREAMDEPENDENCIESREQUEST']._serialized_start=18994
  _globals['_GETUPSTREAMDEPENDENCIESREQUEST']._serialized_end=19190
  _globals['_GETUPSTREAMDEPENDENCIESRESPONSE']._serialized_start=19193
  _globals['_GETUPSTREAMDEPENDENCIESRESPONSE']._serialized_end=19886
  _globals['_METADATA']._serialized_start=20772
  _globals['_METADATA']._serialized_end=26616
  _globals['_API']._serialized_start=26619
  _globals['_API']._serialized_end=32441
# @@protoc_insertion_point(module_scope)



import base64
import hashlib
import os
import threading
from abc import ABC, abstractmethod
from typing import Optional, Tuple

import featureform as ff
import requests
from flask import Flask, request
from typeguard import typechecked
from werkzeug.serving import make_server
import yaml
import time
from okta_jwt_verifier import JWTUtils


@typechecked
class AuthConfig(ABC):
    @abstractmethod
    def get_authorization_endpoint(self, redirect_uri: str, code_challenge: str) -> str:
        pass

    @abstractmethod
    def get_token_exchange_endpoint(self) -> str:
        pass


@typechecked
class OktaAuthConfig(AuthConfig):
    def __init__(
        self, domain: str, authorization_server_id: str, client_id: str
    ) -> None:
        self.domain = domain
        self.authorization_server_id = authorization_server_id
        self.client_id = client_id

    def get_authorization_endpoint(self, redirect_uri: str, code_challenge: str) -> str:
        return (
            f"https://{self.domain}/oauth2/v1/authorize?client_id={self.client_id}"
            f"&response_type=code&scope=openid%20offline_access&redirect_uri={redirect_uri}&state=random_state"
            f"&code_challenge_method=S256&code_challenge={code_challenge}"
        )

    def get_token_exchange_endpoint(self) -> str:
        return f"https://{self.domain}/oauth2/v1/token"

    def get_native_exchange_endpoint(self) -> str:
        return f"https://{self.domain}/oauth2/v1/token?client_id={self.client_id}"
    
    # caveat: uses our custom auth server (enabled by developer account)
    # default org servers don't allow modifying any internal scopes, 
    # and the client credentials flow requires custom auth servers to do so
    def get_machine_exchange_endpoint(self) -> str:
        return f"https://{self.domain}/oauth2/default/v1/token"

    def get_token_refresh_endpoint(self) -> str:
        return f"https://{self.domain}/oauth2/v1/token?client_id={self.client_id}"

@typechecked
class CidrAuthConfig(AuthConfig):
    def __init__(self) -> None:
        pass

    def get_authorization_endpoint(self, redirect_uri: str, code_challenge: str) -> str:
        pass

    def get_token_exchange_endpoint(self) -> str:
        pass


@typechecked
class AuthService(ABC):
    def __init__(self, auth_config) -> None:
        self._auth_config = auth_config

    @abstractmethod
    def authenticate(self) -> None:
        pass

    @abstractmethod
    def refresh_token(self, refresh_token) -> None:
        pass

    @abstractmethod
    def get_access_dict(self) -> Optional[dict]:
        pass

    @abstractmethod
    def clear_access_token(self) -> None:
        pass


@typechecked
class PassThroughService(AuthService):
    def authenticate(self) -> None:
        pass

    def refresh_token(self, _) -> None:
        pass

    def get_access_dict(self) -> Optional[dict]:
        pass

    def clear_access_token(self) -> None:
        pass


@typechecked
class CidrService(AuthService):
    def authenticate(self) -> None:
        pass

    def refresh_token(self, _) -> None:
        pass

    def get_access_dict(self) -> Optional[dict]:
        pass

    def clear_access_token(self) -> None:
        pass


@typechecked
class OktaOAuth2PKCE(AuthService):
    def __init__(self, auth_config: OktaAuthConfig) -> None:
        super().__init__(auth_config)
        # up casts the instance in this class to OktaAuthConfig
        self._auth_config: OktaAuthConfig = auth_config
        self.redirect_uri = "http://localhost:9080/authorization-code/callback"
        self._access_token = None
        self._refresh_token = None
        self._access_token_expires = None
        self._code_verifier = None
        self._callback_server = None
        self._callback_server_thread = None
        self._callback_flask_app = Flask(__name__)
        self._auth_completed = threading.Event()

        @self._callback_flask_app.route("/authorization-code/callback")
        def callback():
            auth_code = request.args.get("code")
            threading.Thread(
                target=self._exchange_code_for_token, args=(auth_code,)
            ).start()
            return "Authentication successful! You can close this window."

    def authenticate(self) -> None:
        self._code_verifier = self._create_code_verifier()
        code_challenge = self._create_code_challenge(self._code_verifier)
        auth_url = self._auth_config.get_authorization_endpoint(
            self.redirect_uri, code_challenge
        )
        print(f"Please visit the following URL to authenticate: {auth_url}")
        self._callback_server_thread = threading.Thread(
            target=self._start_callback_server
        )
        self._callback_server_thread.start()
        self._auth_completed.wait()

    def refresh_token(self, refresh_token) -> None:
        new_token = self._request_refresh_token(refresh_token)

        if new_token:
            print("Token Refreshed!")
        else:
            raise Exception("Failed to refresh access_token")

    def get_access_dict(self) -> Optional[dict]:
        return {
            "access_token": self._access_token,
            "refresh_token": self._refresh_token,
            "access_token_expires": self._access_token_expires,
        }

    def _request_refresh_token(self, refresh_token) -> Optional[str]:
        print("Refreshing Token...")
        headers = {
            "Accept": "application/json",
        }
        data = {
            "grant_type": "refresh_token",
            "refresh_token": refresh_token,
            "scope": "openid offline_access",
        }
        response = requests.post(
            self._auth_config.get_token_refresh_endpoint(),
            headers=headers,
            data=data,
        )

        if response.status_code == 200:
            json_resp = response.json()
            if json_resp:
                self._access_token = json_resp.get("access_token")
                self._refresh_token = json_resp.get("refresh_token")
                self._access_token_expires = int(time.time()) + int(
                    json_resp.get("expires_in")
                )
        else:
            print(f"PKCE failed to get refresh_token: {response.status_code}")
            self._access_token = None
            self._refresh_token = None
            self._access_token_expires = None

        return self._access_token

    def clear_access_token(self) -> None:
        self._auth_completed.clear()
        self._access_token = None
        self._refresh_token = None
        self._access_token_expires = None

    @staticmethod
    def _create_code_verifier():
        token = base64.urlsafe_b64encode(os.urandom(40)).decode("utf-8")
        return token.rstrip("=")

    @staticmethod
    def _create_code_challenge(verifier):
        m = hashlib.sha256()
        m.update(verifier.encode("utf-8"))
        challenge = base64.urlsafe_b64encode(m.digest()).decode("utf-8")
        return challenge.rstrip("=")

    def _exchange_code_for_token(self, auth_code):
        headers = {
            "Content-Type": "application/x-www-form-urlencoded",
            "Accept": "application/json",
        }
        data = {
            "grant_type": "authorization_code",
            "code": auth_code,
            "redirect_uri": self.redirect_uri,
            "client_id": self._auth_config.client_id,
            "code_verifier": self._code_verifier,
            "scope": "openid offline_access",
        }
        try:
            response = requests.post(
                self._auth_config.get_token_exchange_endpoint(),
                headers=headers,
                data=data,
            )
            if response.status_code == 200:
                print("Authentication Succeeded!")
                json_resp = response.json()
                if json_resp:
                    self._access_token = json_resp.get("access_token")
                    self._refresh_token = json_resp.get("refresh_token")
                    self._access_token_expires = self._access_token_expires = int(
                        time.time()
                    ) + int(json_resp.get("expires_in"))
                self._auth_completed.set()
            else:
                raise Exception("Authentication Failed.")
        finally:
            self._stop_callback_server()

    def _start_callback_server(self):
        self._callback_server = make_server("127.0.0.1", 9080, self._callback_flask_app)
        self._callback_server.serve_forever()

    def _stop_callback_server(self):
        if self._callback_server:
            self._callback_server.shutdown()


@typechecked
class OktaOAuthNative(AuthService):
    def __init__(self, auth_config) -> None:
        super().__init__(auth_config)
        self._access_token = None
        self._access_token_expires = None
        self._refresh_token = None

    def authenticate(self) -> None:
        # attempt to use env vars, then cred file
        username = os.environ.get("FF_OKTA_USERNAME")
        password = os.environ.get("FF_OKTA_PASSWORD")

        if username and password:
            print("Using cred vars...")
            self._access_token = self._request_token(
                username=username, password=password
            )
        else:
            print("Using cred file...")
            cred_dict = self._pull_file_creds()
            if cred_dict is not None:
                self._access_token = self._request_token(
                    username=cred_dict.get("username"),
                    password=cred_dict.get("password"),
                )
            else:
                print("No user credentials for okta app found in Environment")
                return None

        if self._access_token:
            print("Authentication Succeeded!")
        else:
            raise Exception("Failed to authenticate with user credentials")

    def refresh_token(self, refresh_token) -> None:
        new_token = self._request_refresh_token(refresh_token)

        if new_token:
            print("Token Refreshed!")
        else:
            raise Exception("Failed to refresh access_token")

    def get_access_dict(self) -> Optional[dict]:
        return {
            "access_token": self._access_token,
            "refresh_token": self._refresh_token,
            "access_token_expires": self._access_token_expires,
        }

    def clear_access_token(self) -> None:
        self._access_token = None
        self._refresh_token = None
        self._access_token_expires = None

    def _request_token(self, username: str, password: str) -> Optional[str]:
        headers = {
            "Accept": "application/json",
        }
        data = {
            "grant_type": "password",
            "username": username,
            "password": password,
            "scope": "openid offline_access",
        }
        response = requests.post(
            self._auth_config.get_native_exchange_endpoint(),
            headers=headers,
            data=data,
        )

        if response.status_code == 200:
            json_resp = response.json()
            if json_resp:
                self._access_token = json_resp.get("access_token")
                self._refresh_token = json_resp.get("refresh_token")
                self._access_token_expires = int(time.time()) + int(
                    json_resp.get("expires_in")
                )
        else:
            print(f"Native failed to get access_token: {response.status_code}")
            try:
                json_resp = response.json()
                print(json_resp)
            except ValueError as e:
                print(f"Failed to parse JSON response: {e}")
            self._access_token = None
            self._refresh_token = None
            self._access_token_expires = None

        return self._access_token

    def _request_refresh_token(self, refresh_token) -> Optional[str]:
        print("Refreshing Token...")
        headers = {
            "Accept": "application/json",
        }
        data = {
            "grant_type": "refresh_token",
            "refresh_token": refresh_token,
            "scope": "openid offline_access",
        }
        response = requests.post(
            self._auth_config.get_token_refresh_endpoint(),
            headers=headers,
            data=data,
        )

        if response.status_code == 200:
            json_resp = response.json()
            if json_resp:
                self._access_token = json_resp.get("access_token")
                self._refresh_token = json_resp.get("refresh_token")
                self._access_token_expires = int(time.time()) + int(
                    json_resp.get("expires_in")
                )
        else:
            print(f"Native failed to get refresh_token: {response.status_code}")
            try:
                json_resp = response.json()
                print(json_resp)
            except ValueError as e:
                print(f"Failed to parse JSON response: {e}")
            self._refresh_token = None
            self._access_token_expires = None

        return self._access_token

    def _pull_file_creds(self) -> Optional[dict]:
        featureform_path = os.environ.get("FEATUREFORM_DIR", ".featureform")
        auth_path = os.path.join(featureform_path, "auth")
        credential_file_path = os.path.join(auth_path, "credentials.yaml")

        if os.path.exists(credential_file_path):
            with open(credential_file_path, "r") as file:
                file_dict = yaml.safe_load(file)
                cred_dict = file_dict.get("okta")
                return cred_dict
        else:
            print("File path does not exist")
            return None

@typechecked
class OktaOAuth2ClientCredentials(AuthService):
    def __init__(self, auth_config) -> None:
        super().__init__(auth_config)
        self._access_token = None
        self._access_token_expires = None

    def authenticate(self) -> None:
        api_client_id = os.environ.get("FF_API_CLIENT_ID")
        api_client_secret = os.environ.get("FF_API_CLIENT_SECRET")

        if not api_client_id or not api_client_secret:
            print("No api client credentials found in Environment, falling back to user and password...")
            return None
        
        auth_header = base64.b64encode(f"{api_client_id}:{api_client_secret}".encode()).decode()

        print("API client and secret set, using client credentials flow...")
        headers = {
            "Authorization": f"Basic {auth_header}",
            "Content-Type": "application/x-www-form-urlencoded",
            "Accept": "application/json",
            "Cache-Control": "no-cache",
        }
        data = {
            "grant_type": "client_credentials",
            "scope": "api.access"
        }
        response = requests.post(
            self._auth_config.get_machine_exchange_endpoint(), headers=headers, data=data
        )


        if response.status_code == 200:
            json_resp = response.json()
            if json_resp:
                self._access_token = json_resp.get("access_token")
                self._access_token_expires = int(time.time()) + int(json_resp.get("expires_in"))
        else:
            print(f"API Credentials failed to get access_token: {response.status_code}")
            try: 
                json_resp = response.json()
                print(json_resp)
            except ValueError as e:
                print(f"Failed to parse JSON response: {e}")
            self._access_token = None
            self._access_token_expires = None
        
        if self._access_token:
            print("Authentication Succeeded!")
        else:
            print("Failed to authenticate with api client credentials")

    def get_access_dict(self) -> Optional[dict]:
        return {
            "access_token": self._access_token,
            "refresh_token": None, # none for api credentials
            "access_token_expires": self._access_token_expires,
        }
    def refresh_token(self, _) -> None:
        print("api client credentials...reauthenticating...")
        self.authenticate()

    def clear_access_token(self) -> None:
        self._access_token = None
        self._access_token_expires = None

@typechecked
class AuthenticationManagerImpl:
    TOKEN_FILENAME = "token.yaml"

    def __init__(self) -> None:
        self._access_token = None
        self._refresh_token = None
        self._access_token_expires = None

        self._auth_config = None
        self._services = []
        feature_form_dir = os.environ.get("FEATUREFORM_DIR", ".featureform")
        self.auth_dir = os.path.join(feature_form_dir, "auth")
        os.makedirs(self.auth_dir, exist_ok=True)
        self.token_filepath = os.path.join(self.auth_dir, self.TOKEN_FILENAME)

    def _write_token_dict_to_file(
        self,
        token_dict: dict,
    ):
        with open(self.token_filepath, "w") as file:
            yaml.dump(token_dict, file)
        os.chmod(self.token_filepath, 0o600)

    def _read_token_dict_from_file(self) -> Optional[dict]:
        if os.path.exists(self.token_filepath):
            with open(self.token_filepath, "r") as file:
                file_dict = yaml.safe_load(file)
                return file_dict
        return {}

    def delete_expired_token(self):
        self._access_token = None
        self._refresh_token = None
        self._access_token_expires = None

        for service in self._services:
            service.clear_access_token()
        feature_form_dir = os.environ.get("FEATUREFORM_DIR", ".featureform")
        auth_dir = os.path.join(feature_form_dir, "auth")

        token_filepath = os.path.join(
            auth_dir, AuthenticationManagerImpl.TOKEN_FILENAME
        )
        if os.path.exists(token_filepath):
            os.remove(token_filepath)

    def get_access_token_or_authenticate(self, insecure, host) -> Optional[dict]:
        token_dict = self._read_token_dict_from_file()
        if token_dict:
            self._access_token = token_dict.get("access_token")
            self._refresh_token = token_dict.get("refresh_token")
            self._access_token_expires = token_dict.get("access_token_expires")

        if self._access_token:
            if (
                isinstance(self._access_token_expires, int)
                and int(time.time()) > self._access_token_expires
            ):
                if not self._services:
                    self._auth_config = self._load_auth_config(insecure, host)
                    if self._auth_config is not None:
                        self._services = [
                            OktaOAuth2ClientCredentials(self._auth_config),
                            OktaOAuthNative(self._auth_config),
                            OktaOAuth2PKCE(self._auth_config)
                        ]
                    else:
                        self._services = [PassThroughService(self._auth_config)]
                for service in self._services:
                    service.refresh_token(self._refresh_token)
                    token_dict = service.get_access_dict()
                    if token_dict and token_dict.get("access_token"):
                        self._access_token = token_dict.get("access_token")
                        self._refresh_token = token_dict.get("refresh_token")
                        self._access_token_expires = token_dict.get(
                            "access_token_expires"
                        )
                        self._write_token_dict_to_file(token_dict)
                        break
        elif not self._access_token:
            if not self._services:
                self._auth_config = self._load_auth_config(insecure, host)
                self._add_services()

            for service in self._services:
                service.authenticate()
                token_dict = service.get_access_dict()
                if token_dict and token_dict.get("access_token"):
                    self._access_token = token_dict.get("access_token")
                    self._refresh_token = token_dict.get("refresh_token")
                    self._access_token_expires = token_dict.get("access_token_expires")
                    self._write_token_dict_to_file(token_dict)
                    break

        resultDict = {
            "token": None,
            "refreshToken": None,
            "subject": None,
        }
        if self._access_token:
            parsed_token = JWTUtils.parse_token(self._access_token)
            if len(parsed_token) != 4:
                raise Exception("Invalid access token, cannot parse the sub property!")
            payload = parsed_token[1]
            resultDict["token"] = self._access_token
            resultDict["refreshToken"] = self._refresh_token
            resultDict["subject"] = payload.get("sub")

        return resultDict

    def get_subject(self, config, insecure, host) -> Optional[str]:
        self._auth_config = self._init_auth_config(config)
        self._add_services()

        if len(self._services) == 1 and isinstance(
            self._services[0], PassThroughService
        ):
            return "default_user"
        elif len(self._services) == 1 and isinstance(self._services[0], CidrService):
            return "allowed_cidr_user"
        else:
            sub = None
            resultDict = self.get_access_token_or_authenticate(insecure, host)
            if resultDict["token"] is not None:
                parsed_token = JWTUtils.parse_token(resultDict["token"])
                if len(parsed_token) != 4:
                    raise Exception("Invalid access token")
                payload = parsed_token[1]
                sub = payload.get("sub")

                # set the appropriate subject from the token
                api_client_id = os.environ.get("FF_API_CLIENT_ID")
                if api_client_id and sub == api_client_id:
                    sub = f"machine-{sub}"
                    print(f"Identified machine user: {sub}")
            return sub

    def _load_auth_config(self, insecure, host) -> Optional[OktaAuthConfig]:
        config = ff.Client(insecure=insecure, host=host).get_auth_config()
        return self._init_auth_config(config)

    def _init_auth_config(self, config) -> Optional[AuthConfig]:
        if config.WhichOneof("config") == "okta":
            okta_config = config.okta
            return OktaAuthConfig(
                domain=okta_config.domain,
                authorization_server_id=okta_config.authorization_server_id,
                client_id=okta_config.client_id,
            )
        elif config.WhichOneof("config") == "cidr":
            return CidrAuthConfig()
        elif config.WhichOneof("config") == "pass_through":
            return None
        else:
            raise Exception("Unsupported auth config")

    def _add_services(self):
        if isinstance(self._auth_config, OktaAuthConfig):
            self._services = [
                OktaOAuth2ClientCredentials(self._auth_config),
                OktaOAuthNative(self._auth_config),
                OktaOAuth2PKCE(self._auth_config)
            ]
        elif isinstance(self._auth_config, CidrAuthConfig):
            self._services = [CidrService(self._auth_config)]
        else:
            self._services = [PassThroughService(self._auth_config)]


singleton = AuthenticationManagerImpl()
